/**
 * 工具类
 */
_ = {};
_.escape = (function() {
    var map = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#x27;',
        '`': '&#x60;'
    };
    var escaper = function(match) {
        return map[match];
    };
    var source = '(?:' + Object.keys(map).join('|') + ')';
    var testRegexp = RegExp(source);
    var replaceRegexp = RegExp(source, 'g');
    return function(string) {
        string = string == null ? '' : '' + string;
        return testRegexp.test(string) ? string.replace(replaceRegexp, escaper) : string;
    };
})();
_.isFunction = function(obj) {
    return Object.prototype.toString.call(obj) === '[object Function]';
}
if (typeof /./ != 'function' && typeof Int8Array != 'object') {
    _.isFunction = function(obj) {
      return typeof obj == 'function' || false;
    };
}
_.has = function(obj, key) {
  return obj != null && Object.prototype.hasOwnProperty.call(obj, key);
};

// Internal recursive comparison function for `isEqual`.
var eq = function(a, b, aStack, bStack) {
  // Identical objects are equal. `0 === -0`, but they aren't identical.
  // See the [Harmony `egal` proposal](http://wiki.ecmascript.org/doku.php?id=harmony:egal).
  if (a === b) return a !== 0 || 1 / a === 1 / b;
  // A strict comparison is necessary because `null == undefined`.
  if (a == null || b == null) return a === b;
  // Unwrap any wrapped objects.
  // if (a instanceof _) a = a._wrapped;
  // if (b instanceof _) b = b._wrapped;
  // Compare `[[Class]]` names.
  var className = toString.call(a);
  if (className !== toString.call(b)) return false;
  switch (className) {
    // Strings, numbers, regular expressions, dates, and booleans are compared by value.
    case '[object RegExp]':
    // RegExps are coerced to strings for comparison (Note: '' + /a/i === '/a/i')
    case '[object String]':
      // Primitives and their corresponding object wrappers are equivalent; thus, `"5"` is
      // equivalent to `new String("5")`.
      return '' + a === '' + b;
    case '[object Number]':
      // `NaN`s are equivalent, but non-reflexive.
      // Object(NaN) is equivalent to NaN
      if (+a !== +a) return +b !== +b;
      // An `egal` comparison is performed for other numeric values.
      return +a === 0 ? 1 / +a === 1 / b : +a === +b;
    case '[object Date]':
    case '[object Boolean]':
      // Coerce dates and booleans to numeric primitive values. Dates are compared by their
      // millisecond representations. Note that invalid dates with millisecond representations
      // of `NaN` are not equivalent.
      return +a === +b;
  }

  var areArrays = className === '[object Array]';
  if (!areArrays) {
    if (typeof a != 'object' || typeof b != 'object') return false;

    // Objects with different constructors are not equivalent, but `Object`s or `Array`s
    // from different frames are.
    var aCtor = a.constructor, bCtor = b.constructor;
    if (aCtor !== bCtor && !(_.isFunction(aCtor) && aCtor instanceof aCtor &&
                             _.isFunction(bCtor) && bCtor instanceof bCtor)
                        && ('constructor' in a && 'constructor' in b)) {
      return false;
    }
  }
  // Assume equality for cyclic structures. The algorithm for detecting cyclic
  // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.

  // Initializing stack of traversed objects.
  // It's done here since we only need them for objects and arrays comparison.
  aStack = aStack || [];
  bStack = bStack || [];
  var length = aStack.length;
  while (length--) {
    // Linear search. Performance is inversely proportional to the number of
    // unique nested structures.
    if (aStack[length] === a) return bStack[length] === b;
  }

  // Add the first object to the stack of traversed objects.
  aStack.push(a);
  bStack.push(b);

  // Recursively compare objects and arrays.
  if (areArrays) {
    // Compare array lengths to determine if a deep comparison is necessary.
    length = a.length;
    if (length !== b.length) return false;
    // Deep compare the contents, ignoring non-numeric properties.
    while (length--) {
      if (!eq(a[length], b[length], aStack, bStack)) return false;
    }
  } else {
    // Deep compare objects.
    var keys = Object.keys(a), key;
    length = keys.length;
    // Ensure that both objects contain the same number of properties before comparing deep equality.
    if (Object.keys(b).length !== length) return false;
    while (length--) {
      // Deep compare each member
      key = keys[length];
      if (!(_.has(b, key) && eq(a[key], b[key], aStack, bStack))) return false;
    }
  }
  // Remove the first object from the stack of traversed objects.
  aStack.pop();
  bStack.pop();
  return true;
};

_.isEqual = function(a, b) {
  return eq(a, b);
};

/**
 *  window.Press_State({});
 *  bindSelector    绑定元素选择器 必填  '.movie_detail,.comment'
 *  exceptSelector  排除元素选择器 选填  默认为空
 *  pressedClass    按压效果类   选填  默认为pressed 参照UI面四按压效果
 *  triggerLatency  延时触发时间  选填  默认100毫秒
 *  removeLatency   跳转后移除按压效果时间 选填 默认为100毫秒
 */
(function($) {
    var globals = {};

    function press_state(options) {
        var defaults = {
            bindSelector: "",
            exceptSelector: "",
            pressedClass: "pressed",
            triggerLatency: 100,
            removeLatency: 100
        };
        this.settings = $.extend({}, defaults, options);
        this._init();
    }
    press_state.prototype = {
        _init: function() {
            if (this.settings.bindSelector == "")
                return;

            this._appendClass();
            this._bindEvent();
        },
        _appendClass: function() {
            if (this.settings.pressedClass == "pressed") {
                var style = "<style type='text/css'>.pressed{background-color: #e0e0e0 !important;} .night .pressed{background-color: #1b1b1b !important;}</style>";
                $('body').append(style);
            }
        },
        _bindEvent: function() {
            var holder = 'body';
            var element = this.settings.exceptSelector == '' ? this.settings.bindSelector : [this.settings.bindSelector, this.settings.exceptSelector].join(',');
            var except = this.settings.exceptSelector;
            var pressedClass = this.settings.pressedClass;
            var triggerl = parseInt(this.settings.triggerLatency);
            var removel = parseInt(this.settings.removeLatency);
            $(holder).on('touchstart', element, function(event) {
                if (!$(this).is(except)) {
                    var $this = $(this);
                    globals.mytimer = setTimeout(function() {
                        $this.addClass(pressedClass);
                    }, triggerl);

                    globals.tar = event.target;
                }
            })
            $(holder).on('touchmove', element, function(event) {
                if (!$(this).is(except)) {
                    clearTimeout(globals.mytimer);
                    $(this).removeClass(pressedClass);

                    globals.tar = null;
                }
            })
            $(holder).on('touchend touchcancel', element, function(event) {
                if (!$(this).is(except)) {
                    if (globals.tar === event.target) {
                        clearTimeout(globals.mytimer);
                        if (!$(this).hasClass(pressedClass)) {
                            $(this).addClass(pressedClass);
                        }
                        var $now = $(this);
                        setTimeout(function() {
                            $now.removeClass(pressedClass);
                        }, removel);
                    }
                }
            })
        }
    }
    $.Press_State = function(options) {
        new press_state(options);
    }
})(window.jQuery || window.Zepto);


/**
 * 事务均执行成功后的回调
 *
 * @callback assureSuccessCallback
 * @param {object} alldata 各事务返回数据对象，以事务名做key
 */
/**
 * 某一事务执行失败后的回调
 *
 * @callback assureSuccessCallback
 * @param {string} transactionName 失败的事务名
 * @param {*} error 失败的信息或数据
 */
/**
 * 事务调度
 * @param {string[]} transactions 事务名称字符串数组
 * @param {assureSuccessCallback} success 事务均执行成功后的回调
 * @param {assureErrorCallback} error 某一事务执行失败后的回调
 * @param {...*} params 传入各事务的数据
 */
function Assure(transactions, success, error, params) {
    this.transactions = transactions;
    this.success = success;
    this.error = error;

    this.failTransactions = [];
    this.realData = {};

    var context = window;
    var that = this;

    params = Array.prototype.slice.call(arguments, 3);
    params.unshift(that);

    this.transactions.forEach(function(tn) {
        if (typeof context[tn] === 'function') {
            context[tn].apply(context, params);
        }
    });
}
Assure.prototype = {
    constructor: Assure,
    checkIfAllResolved: function() {
        for (var i in this.transactions) {
            if (!(this.transactions[i] in this.realData)) {
                return false;
            }
        }
        return true;
    },
    addRealArgs: function(transactions, data) {
        this.realData[transactions] = data;
    },
    resolve: function(transactions, data) {
        this.addRealArgs(transactions, data);
        if (this.checkIfAllResolved()) {
            this.success(this.realData);
        }
    },
    reject: function(transactions, error) {
        this.addRealArgs(transactions, error);
        this.failTransactions.push(transactions);
        if (this.checkIfAllResolved()) {
            this.error(this.failTransactions, this.realData);
        }
    }
};

/**
 * 构造统一Vicon展示
 */
function buildServerVIcon(auth, icon, extraClass) {
    var q = globalArticleObject.h5_settings.user_verify_info_conf['' + auth];
    if (!q) {
        return '';
    }
    q = q[icon];
    if (!q) {
        return '';
    }

    // NOTE
    // 安卓全系(4.4+)用webp，
    // ios全系用apng(7+会fallback为静态图)
    var icon = q.icon;
    if (is_iphone()) {
        icon = q.web_icon_ios + '?' + Date.now();
    } else if (is_android_version_bigger_than('4.4')) {
        icon = q.web_icon_android;
    } else {
        icon = q.icon_png;
    }

    if (!extraClass) {
        extraClass = '';
    }
    return '<i class="server-v-icon ' + extraClass + '" style="background-image: url(' + icon + '); width: ' + q.width/3 + ';">&nbsp;</i>';
}

/**
 * 服务端传过来是数组，将其转换成type做key的对象
 */
function trans_v_info(json) {
    var ret = {};
    if (Array.isArray(json.type_config)) {
        for (var i = 0; i < json.type_config.length; i++) {
            var item = json.type_config[i];
            ret[item.type] = item;
        }
    }
    return ret;
}
;

/**
 * 统计工具
 */
/**
 * umeng统计
 */
var is_android_bool = /android/i.test(navigator.userAgent);
var event_type = is_android_bool ? 'log_event' : 'custom_event';

/**
 * 发送umeng统计
 * @param  {string} tag   tag
 * @param  {string} label label
 * @param  [object] sobj  可能后缀的统计字段
 * @return {undefined}
 */
function send_umeng_event (tag, label, sobj) {
    var s = 'bytedance://' + event_type + '?category=umeng&tag=' + tag + '&label=' + label;

    if (sobj) {
        for (var sitem in sobj) {
            var svalue = sobj[sitem];
            if (sitem === 'extra' && typeof svalue === 'object') {
                if (is_android_bool) {
                    s += '&extra=' + JSON.stringify(svalue);
                } else {
                    var e = '';
                    for (var eitem in svalue) {
                        if (typeof svalue[eitem] === 'object') {
                            e += '&' + eitem + '=' + JSON.stringify(svalue[eitem]);
                        } else {
                            e += '&' + eitem + '=' + svalue[eitem];
                        }
                    }
                    s += e;
                }
            } else {
                s += '&' + sitem + '=' + svalue;
            }
        }
    }

    // NOTE console.log也能发统计
    //      https://wiki.bytedance.com/pages/viewpage.action?pageId=20777525
    console.log(s);
    // location.href = s;
}

/**
 * 向客户端发送bytedance://请求，如加载各种类型图片
 * @param {string} protocol 协议类型
 * @param [object] params 传递给客户端的参数对象（可选，部分协议类型不需要此参数）
 */
function send_request (protocol, params) {
    var s = 'bytedance://' + protocol;
    if (params) {
        s += '?';
        for (var field in params) {
            s += (field + '=' + params[field] + '&');
        }
        s = s.slice(0, -1);
    }
    location.href = s;
}

/**
* show事件，相关DOM区域滚动到屏幕中可见时仅发送一次友盟事件
* @param {object} target 目标DOM
* @param {function} event_handle 目标元素露出后的处理函数（发送友盟事件）
* @param {boolean} shownTotally 是否DOM底部完全露出后才发送事件
*/
function send_exposure_event_once(target, event_handle, shownTotally){
    if(!target || typeof event_handle != 'function') return;

    var scrollTimer     = 0,
        viewHeight      = window.innerHeight;

    if( is_inview(target, viewHeight) ){
         event_handle();
     }else{
        document.addEventListener("scroll", page_scroll, false);
    }

    function page_scroll(){
        if (scrollTimer) {
            clearTimeout(scrollTimer);
        }
        scrollTimer = setTimeout(function () {
            var flag = is_inview(target, viewHeight);
            console.info(flag, target);
            if( flag ){
                event_handle();
                document.removeEventListener("scroll", page_scroll, false);
            }
        }, 50);
    }

    function is_inview(con, viewHeight){
        var conRect     = con.getBoundingClientRect(),
            conTop      = conRect.top,
            conHeight   = conRect.height || conRect.bottom - conRect.top,
            _baseline   = conTop;

        if(shownTotally){
            _baseline = conTop + conHeight;
        }
        return _baseline < viewHeight;
    }
}
;

/**
 * 基础工具
 */
var hash = (function () {
    // e.g. #tt_image=origin&tt_font=m&tt_daymode=1
    var hash = location.hash.substr(1);
    var hashObject = {};

    if (hash) {
        hash.split('&').forEach(function(query){
            query = query.split('=');
            var field = query[0];
            var value = query[1];
            if (field) {
                hashObject[field] = value;
            }
        });
    }

    /**
     * 获取、设置页面hash
     * @param  {string|object} field hash的key
     * @param  {string|undefined} value hash的value
     * @return {string|undefined}
     *  hash('tt_image') => 'origin'
     *  hash('tt_font', 'xl') => tt_font=xl
     *  hash({'tt_image': 'none', 'tt_daymode': 0}) => tt_image=none&tt_daymode=0
     */
    return function (field, value) {
        var newHashObject = {};
        if (field === undefined && value === undefined) {
            return location.hash;
        }
        if (value === undefined && typeof field === 'string') {
            return hashObject[field];
        } else if (typeof field === 'string' && typeof value === 'string') {
            newHashObject[field] = value;
        } else if (value === undefined && typeof field === 'object') {
            newHashObject = field;
        }
        $.extend(hashObject, newHashObject);
        location.hash = hash2string(hashObject);
    };
})();

/**
 * 将hash从Object形式转换成string
 * @param  {object} hashObject hash对象
 * @return {string} hash字符串
 */
function hash2string (hashObject) {
    var hash = '#';
    for (var field in hashObject) {
        hash += field + '=' + hashObject[field]+ '&';
    }
    if (hash.substr(-1) == '&') {
        hash = hash.slice(0, -1);
    } else if (hash.substr(-1) == '#') {
        hash = '';
    }
    return hash;
}


/**
 * 获取页面meta信息
 */
var getMeta = (function () {
    var domMetas = document.getElementsByTagName('meta');
    var metasObj = {};
    for (var i = 0, len = domMetas.length; i < len; i++) {
        var name = domMetas[i].name.toLowerCase();
        var content = domMetas[i].getAttribute('content');
        if (name && content) {
            metasObj[name] = content;
        }
    }

    return function (name) {
        return metasObj[name];
    }
})();

/**
 * 获取location.search中的指定参数的值
 * @param {string} params 待查询参数
 */
function request (params) {
    var s = location.search.substr(1);
    var paraObj = {};

    if (s) {
        var arr = s.split('&');
        for (var i = 0; i< arr.length; i++) {
            var t = arr[i].split('=');
            paraObj[t[0]] = t[1];
        }
    }
    return paraObj[params.toLowerCase()];
}

/**
 * 判断视频是否处于屏幕可视区域
 * @param {object} element DOM
 * @return {boolean} 元素是否处于可视区域
 */
function _videoInView (element) {
    var coords = element.getBoundingClientRect();
    var video_height = coords.height || 100;
    return ((coords.top >= 0 && coords.left >= 0 && coords.top) <= (window.innerHeight || document.documentElement.clientHeight) - video_height);
}

/**
 * 格式化显示顶／赞／踩数量
 * @param  {string} selector 显示容器选择器
 * @param  {number} realnum  原始数值
 * @param  {string} placeholder   当原始数值为0时默认显示的字段
 * @return {string} 返回格式化后的数据
 */
function formatCount(selector, realnum, placeholder) {
    var formatnum = '';

    if (typeof realnum !== 'number' || realnum === 0) {
        formatnum = placeholder || '赞';
    } else if (realnum < 1e4) {
        formatnum = realnum;
    } else if (realnum < 1e8) {
        var d = (Math.floor(realnum / 1e3) / 10).toFixed(1);
        formatnum = (d.indexOf('.0') > -1 || d >= 10 ? d.slice(0, -2) : d) + '万';
    } else {
        var d = (Math.floor(realnum / 1e7) / 10).toFixed(1);
        formatnum = (d.indexOf('.0') > -1 || d >= 10 ? d.slice(0, -2) : d) + '亿';
    }

    if (selector) {
        $(selector).each(function(){
            $(this).attr('realnum', realnum).html(formatnum);
        });
    }

    return formatnum;
}


/**
 * 将服务端传过来的时间戳（秒）转为格式化的字符串
 * @param  {string|number} secondsTimestamp 秒级时间戳
 * @return {string}  格式化时间
 */
function commentTimeFormat(secondsTimestamp) {
    var now = new Date();
    var ret = '';
    var d;

    try {
        d = new Date(secondsTimestamp * 1000);
        if (isNaN(d.getTime())) {
            throw new Error('Invalid Date');
        }
    } catch(err) {
        return '';
    }

    ret += d.getFullYear() < now.getFullYear() ? (d.getFullYear() + '-') : '';
    ret += d.getMonth() >= 9 ? (d.getMonth() + 1) : ('0' + (d.getMonth() + 1));
    ret += '-';
    ret += d.getDate() > 9 ? d.getDate() : ('0' + d.getDate());
    ret += ' ';
    ret += d.getHours() > 9 ? d.getHours() : ('0' + d.getHours());
    ret += ':';
    ret += d.getMinutes() > 9 ? d.getMinutes() : ('0' + d.getMinutes());

    return ret;
}

/**
 * 将服务端传过来的时间戳（毫秒）转为格式化的字符串
 * @param  {string|number} millisecondTimestamp 毫秒级时间戳
 * @return {string}  格式化时间
 */
function formatTime(millisecondTimestamp) {
    var ONE_MINUTE = 1000 * 60;
    var ONE_HOUR = ONE_MINUTE * 60;
    var ONE_DAY = ONE_HOUR * 24;

    var now = new Date();
    var nowTimestamp = now.getTime();
    var fakeDate = new Date(now.getFullYear(), now.getMonth(), now.getDate());
    var d = new Date(+millisecondTimestamp);

    if (isNaN(d.getTime())) {
        return '';
    }

    var delta = nowTimestamp - millisecondTimestamp;

    if (delta < 0) {
        return '';
    } else if (delta < ONE_MINUTE) {
        return '刚刚';
    } else if (delta < ONE_HOUR) {
        return Math.floor(delta / ONE_MINUTE) + '分钟前';
    } else if (delta < 24 * ONE_HOUR) {
        return Math.floor(delta / ONE_HOUR) + '小时前';
    } else {
        var _hm = (d.getHours() > 9 ? d.getHours() : ('0' + d.getHours())) + ':' + (d.getMinutes() > 9 ? d.getMinutes() : ('0' + d.getMinutes()));

        var i = 0;
        while (i++ <= 8) {
            fakeDate.setDate(fakeDate.getDate() - 1);
            if (millisecondTimestamp > fakeDate.getTime()) {
                if (i === 1) {
                    return '昨天 ' + _hm;
                } else if (i === 2) {
                    return '前天 ' + _hm;
                } else {
                    return i + '天前';
                }
            }
        }

        return (d.getFullYear() < now.getFullYear() ? (d.getFullYear() + '年') : '') + (d.getMonth() + 1) + '月' + d.getDate() + '日';
    }
}


/**
 * 获取三位版本号
 * @return {undefined}
 */
function get_app_version () {
    var matches = /NewsArticle\/(\d\.\d\.\d)/i.exec(navigator.userAgent);
    if (matches) {
        return matches[1];
    }
    return '';
}

/**
 * 只取两位版本号做对比
 * @param  {number}  n 要对比的版本号
 * @return {Boolean}
 */
function is_android_version_bigger_than (n) {
    var ua = navigator.userAgent.toLowerCase();
    if (ua.indexOf('android') > -1) {
        var androidVersion = parseFloat(ua.substr(ua.indexOf('android') + 8, 3));
        return androidVersion >= n;
    } else {
        return true;
    }
}

function is_iphone() {
    return navigator.userAgent.toLowerCase().indexOf('iphone') > -1;
}

function is_android() {
    return navigator.userAgent.toLowerCase().indexOf('android') > -1;
}

/**
 * 判断应用版本号是否大于（等于）指定版本
 * @param  {String}  version 用dot分隔的版本号，最多支持三位版本
 * @return {Boolean}
 */
function is_app_version_bigger_than (version) {
    var currentVersion = get_app_version();
    if (!currentVersion) {
        return false;
    }
    version = version.split('.').slice(0, 3);
    currentVersion = +currentVersion.split('.').slice(0, version.length).join('');

    return currentVersion >= (+version.join(''));
}

/**
 * 将秒数格式化位xx:xx
 * @param  {number} duration 秒
 * @return {string} 格式化后的字符串
 */
function formatDuration (duration) {
    if (isNaN(Number(duration))) {
        return '00:00';
    }
    var d = [Math.floor(duration / 60), ':', Math.ceil(duration % 60)];
    d[2] <= 9 && d.splice(2, 0, 0);
    d[0] <= 9 && d.unshift(0);
    return d.join('');
}

var sendBytedanceRequest = (function () {
    var _iframeID = 'SEND-BYTE--DANCE-REQUEST';
    var _iframe = document.getElementById(_iframeID);

    if (!_iframe) {
        _iframe = document.createElement('iframe');
        _iframe.id = _iframeID;
        _iframe.style.display = 'none';
        document.body.appendChild(_iframe);
    }

    return function(url) {
        _iframe.src = url;
    }
})();
;

/**
 * 客户端调用渲染头尾部
 * TODO 有严重的bug隐患！在domcontentload事件之前调用是不对的！
 *      这个try也不太懂
 */
window.globalArticleObject = {};

function mergeArguments() {
    var defaults = {
    // required
        article: {},
        author: {},
        tags: [],
        h5_settings: {},
        statistics: {}
    // optional
        // wenda_extra  {+id,+aniok}
        // forum        {name,link,readCount}
        // original     {link,name}
        // menu         passthrough
        // zz_comments  passthrough
        // novel_data   passthrough
    };

    var extraProcessor = {
        getArticleType: function getArticleType() {
            var _type = 'zhuanma';

            if ('wenda_extra' in window) {
                _type = 'wenda';
            } else if ('forum_extra' in window) {
                _type = 'forum';
            } else if (typeof h5_extra.media === 'object' && h5_extra.media !== null && h5_extra.media.id != 0) {
                _type = 'pgc'
            }

            return _type;
        },
        wenda: function wenda() {
            // NOTE 1. 发布时间在最底部
            //      2. isAuthorSelf通过set_info下发，后改为infor接口
            //      3. 无需展示认证信息，只显示intro即可（已后端拼好），认证信息用于展示v
            //      4. 安卓4.4以下不展示按钮动画，首次进页面也不展示
            var _user = wenda_extra.user || {};

            defaults.article = {
                title: wenda_extra.title,
                publishTime: wenda_extra.show_time
            };

            defaults.author = {
                userId: _user.user_id,
                name: _user.user_name,
                link: defaults.h5_settings.is_liteapp ? 'javascript:;' : _user.schema,
                intro: _user.user_intro,
                avatar: _user.user_profile_image_url,
                isAuthorSelf: false, // 问答的isAuthorSelf通过set_info/wenda_context接口下发
                verifiedContent: _user.is_verify ? 'PLACE_HOLDER' : ''
            };

            var _user_auth_info = {
                auth_type: '',
                auth_info: ''
            };
            try {
                _user_auth_info = JSON.parse(_user.user_auth_info);
            } catch (ex) {
                // just do nothing.
            }
            defaults.author.auth_type = _user.user_auth_info ? (_user_auth_info.auth_type || 0) : '';
            defaults.author.auth_info = _user_auth_info.auth_info;

            // 为了加快关注状态展示速度，客户端会传递is_following字段
            if ('is_following' in wenda_extra) {
                defaults.author.followState = wenda_extra.is_following ? 'following' : '';
            }

            defaults.wenda_extra = wenda_extra;
            // defaults.wenda_extra.aniok = Utils.client.isAndroid && Utils.client.osVersion >= '4.4';
            defaults.wenda_extra.aniok = is_android_version_bigger_than(4.4);
        },
        forum: function forum() {
            // NOTE 1. 关注状态通过set_info下发
            //      2. 简介使用头条号名字＋认证信息
            var _user_info = forum_extra.user_info || {};
            var _forum_info = forum_extra.forum_info || {};

            defaults.article = {
                title: forum_extra.thread_title || '',
                publishTime: formatTime(forum_extra.publish_timestamp*1000)
            };

            defaults.author = {
                userId: _user_info.id,
                name: _user_info.name,
                link: _user_info.schema,
                avatar: _user_info.avatar_url,
                isAuthorSelf: !!forum_extra.is_author,
                verifiedContent: _user_info.verified_content
            };

            var _user_auth_info = {
                auth_type: '',
                auth_info: ''
            };
            try {
                _user_auth_info = JSON.parse(_user_info.user_auth_info);
            } catch (ex) {
                // just do nothing.
            }
            defaults.author.auth_type = _user_info.user_auth_info ? (_user_auth_info.auth_type || '0') : '';
            defaults.author.auth_info = _user_auth_info.auth_info;

            // 为了加快关注状态展示速度，客户端会传递is_following字段
            if ('is_following' in forum_extra) {
                defaults.author.followState = forum_extra.is_following ? 'following' : '';
            }

            var _intros = [];
            if (typeof _user_info.media === 'object' && _user_info.media.name) {
                _intros.push(_user_info.media.name);
            }
            if (_user_info.verified_content) {
                _intros.push(_user_info.verified_content);
            }
            defaults.author.intro = _intros.join('，');

            defaults.tags = forum_extra.label_list;

            defaults.forum_extra = forum_extra;

            defaults.forumStatisticsParams = {
                value: forum_extra.thread_id,
                ext_value: forum_extra.forum_id,
                extra: {
                    enter_from: forum_extra.enter_from,
                    concern_id: forum_extra.concern_id,
                    refer: forum_extra.refer,
                    group_type: forum_extra.group_type,
                    category_id: forum_extra.category_id
                }
            }
        },
        pgc: function pgc() {
            var _media = h5_extra.media || {};

            defaults.article = {
                title: h5_extra.title,
                publishTime: h5_extra.publish_stamp ? formatTime(h5_extra.publish_stamp * 1000) : h5_extra.publish_time
            };

            defaults.author = {
                userId: h5_extra.media_user_id,
                mediaId: _media.id, // 作者、用户统一
                name: _media.name,
                link: 'sslocal://profile?refer=all&source=article_top_author&uid=' + h5_extra.media_user_id,
                intro: _media.description,
                avatar: _media.avatar_url,
                isAuthorSelf: !!h5_extra.is_author
            };
            // lite版有莫名的问题，要兼容一下；对于线上老版本，暂时忽略
            if (defaults.h5_settings.is_liteapp || !h5_extra.media_user_id) {
                defaults.author.link = 'bytedance://media_account?refer=all&media_id=' + _media.id + '&loc=0&entry_id=' + _media.id;
            }

            var _user_auth_info = {
                auth_type: '',
                auth_info: ''
            };
            try {
                _user_auth_info = JSON.parse(_media.user_auth_info);
            } catch (ex) {
                // just do nothing.
            }
            defaults.author.auth_type = _media.user_auth_info ? (_user_auth_info.auth_type || 0) : '';
            defaults.author.auth_info = _user_auth_info.auth_info;

            defaults.author.verifiedContent = _media.user_verified && defaults.author.auth_info || '';

            // 为了加快关注状态展示速度，客户端会传递is_subscribed字段
            if ('is_subscribed' in h5_extra) {
                defaults.author.followState = h5_extra.is_subscribed ? 'following' : '';
            }

            // 原创标签
            if (h5_extra.is_original) {
                defaults.tags.push('原创');
            }

            // 尝试做一次parse，如果传递了menu信息会被添加上
            try {
                defaults.menu = JSON.parse(_media.pgc_custom_menu);
            } catch (ex) {
                // just do nothing.
            }
        },
        zhuanma: function zhuanma() {
            // NOTE 转码页展示时间、作者和查看原文入口即可
            defaults.article = {
                title: h5_extra.title,
                originalLink: h5_extra.src_link,
                publishTime: h5_extra.publish_time || '0000-00-00 00:00'
            };

            defaults.author.name = h5_extra.source;
        },
        common: function common() {
            // 转自头条号：xxx
            if (!h5_extra.is_original && h5_extra.original_media_id) {
                defaults.original = {
                    link: 'bytedance://media_account?media_id=' + h5_extra.original_media_id + '&entry_id=' + h5_extra.original_media_id,
                    name: h5_extra.original_media_name || ''
                };
            }

            // 转载推荐
            if ('zz_comments' in window) {
                defaults.zz_comments = zz_comments;
            }

            // 小说
            if ('novel_data' in h5_extra) {
                defaults.novel_data = h5_extra.novel_data;
            }

            // NOTE 客户端AB测试关注按钮［关注／订阅］状态，f7表示为‘关注’版本，该情况下，详情页
            //      顶部展示‘＋关注’按钮；其他情况下，展示为‘＋订阅’按钮
            //      https://wiki.bytedance.com/pages/viewpage.action?pageId=13966145
            var _ab_client = h5_extra.ab_client || [];
            if (defaults.article.type !== 'pgc' || _ab_client.indexOf('f7') > -1) {
                defaults.topbuttonType = 'concern';
            } else {
                defaults.topbuttonType = 'digg';
            }

            // 跟人相关的ab测试，iOS传过来是字典，android是json字符串
            try {
                if (typeof h5_extra.h5_settings === 'object') {
                    defaults.h5_settings = h5_extra.h5_settings;
                } else {
                    defaults.h5_settings = JSON.parse(h5_extra.h5_settings);
                }
            } catch (ex) {
                defaults.h5_settings = {};
            }
            // 确保头部置换只发生在pgc文章上
            defaults.h5_settings.pgc_over_head = !!defaults.h5_settings.pgc_over_head && defaults.article.type === 'pgc';
            // lite版很多幺蛾子
            defaults.h5_settings.is_liteapp = !!h5_extra.is_lite;
            // defaults.h5_settings.pgc_author_card = true;
            // 前端开全量
            defaults.h5_settings.pgc_recommend_connect_fe = true;
            // 从客户端接图标，接到了按新的要求做，否则维持原状
            if (defaults.h5_settings.user_verify_info_conf) {
                if (typeof defaults.h5_settings.user_verify_info_conf === 'string') {
                    try {
                        defaults.h5_settings.user_verify_info_conf = JSON.parse(defaults.h5_settings.user_verify_info_conf);
                    } catch (ex) {
                        defaults.h5_settings.user_verify_info_conf = {};
                    }
                }
                defaults.h5_settings.user_verify_info_conf = trans_v_info(defaults.h5_settings.user_verify_info_conf);
                defaults.useServerV = true;
            } else {
                defaults.useServerV = false;
            }

            // 非关注推荐文章减少关注按钮占用空间，此变量在后续代码中可替代「关注推荐控制开关」
            defaults.hasExtraSpace = !defaults.h5_settings.is_liteapp && defaults.h5_settings.pgc_recommend_connect_fe && defaults.article.type === 'pgc' && is_android_version_bigger_than(4.4);

            // 导流页『升级』不显示关注按钮
            defaults.hideFollowButton = !!h5_extra.hideFollowButton;

            defaults.statistics = {
                group_id: h5_extra.str_group_id || h5_extra.group_id || '',
                item_id: h5_extra.str_item_id || h5_extra.item_id || ''
            };
        }
    };

    if (typeof window.h5_extra === 'object') {
        var _articletype = extraProcessor.getArticleType();
        defaults.article.type = _articletype;
        extraProcessor.common();
        extraProcessor[_articletype]();
        defaults.article.type = _articletype; // 分类的processor会覆盖atricle变量，要重写一次
    }

    return defaults;
}

var HeaderTemplateFunction = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<header topbutton-type="'+
((__t=( data.topbuttonType ))==null?'':__t)+
'" sugstate="no" '+
((__t=( data.hasExtraSpace ? "" : "no-extra-space" ))==null?'':__t)+
'>';
 /* 转载部分 */ 
__p+='';
 if (data.zz_comments) { 
__p+='<div class="zzcomments"><span class="rec-us"></span><span class="rec-end" style="visibility: hidden;">推荐此文</span></div>';
 } 
__p+='';
 /* PGC作者置顶测试 */ 
__p+='';
 if (!data.h5_settings.pgc_over_head) { 
__p+='<div class="tt-title">'+
((__t=( data.article.title ))==null?'':__t)+
'</div>';
 } 
__p+='';
 /* 作者部分 */ 
__p+='';
 if (data.article.type == 'zhuanma') { 
__p+='<div class="zhuanma-wrapper"><span class="zm-time">'+
((__t=( data.article.publishTime ))==null?'':__t)+
'</span><span class="zm-author">'+
((__t=( data.author.name ))==null?'':__t)+
'</span>';
 if (data.article.originalLink) { 
__p+='/<a class="original-link" href="'+
((__t=( data.article.originalLink ))==null?'':__t)+
'">查看原文</a>';
 } 
__p+='</div>';
 } else { 
__p+='<div class="authorbar '+
((__t=( data.article.type ))==null?'':__t)+
'" id="profile">';
 /* 头像 */ 
__p+='<a class="author-avatar-link pgc-link" href="'+
((__t=( data.author.link ))==null?'':__t)+
'"><img class="author-avatar" src="'+
((__t=( data.author.avatar ))==null?'':__t)+
'"></a>';
 /* 作者可见的阅读信息 */ 
__p+='';
 if (data.article.type === 'wenda') { 
__p+='<div class="wenda-info" style="display: '+
((__t=( data.author.isAuthorSelf ? 'block' : 'none' ))==null?'':__t)+
';"><span class="read-info brow-count"></span><span class="like-info digg-count-special"></span></div>';
 } else if (data.article.type === 'forum') { 
__p+='<div class="wenda-info" style="display: '+
((__t=( data.author.isAuthorSelf ? 'block' : 'none' ))==null?'':__t)+
';"><span></span></div>';
 } 
__p+='';
 /* 关注按钮和关注推荐 */ 
__p+='<div class="author-function-buttons"><div class="mediasug-arrow-button iconfont"></div><a class="subscribe-button follow-button '+
((__t=( 'followState' in data.author ? data.author.followState : 'disabled'))==null?'':__t)+
'"data-user-id="'+
((__t=( data.author.userId ))==null?'':__t)+
'"data-media-id="'+
((__t=( data.author.mediaId ))==null?'':__t)+
'"href="javascript:;"style="display: '+
((__t=( data.author.isAuthorSelf || (data.article.type === 'wenda' && data.h5_settings.is_liteapp) || (data.article.type === 'forum' &&  data.author.followState === 'following') || data.hideFollowButton ? 'none' : 'block' ))==null?'':__t)+
';"id="subscribe"><i class="iconfont focusicon">&nbsp;</i></a></div>';
 /* 作者和文章信息 */ 
__p+='<div class="author-bar"><div class="name-link-w '+
((__t=( (data.article.type === 'wenda' && data.author.intro === '' && data.tags.length === 0) ? 'no-intro' : '' ))==null?'':__t)+
'"><a class="author-name-link pgc-link" href="'+
((__t=( data.author.link ))==null?'':__t)+
'">'+
((__t=( data.author.name ))==null?'':__t)+
'</a>';
 if (data.useServerV) { 
__p+='';
 if (data.author.auth_info) { 
__p+=''+
((__t=( buildServerVIcon(data.author.auth_type, 'label_icon') ))==null?'':__t)+
'';
 } 
__p+='';
 } else { 
__p+='';
 if (data.author.verifiedContent != '') { 
__p+='<div class="iconfont verified-icon">&#xe600;</div>';
 } 
__p+='';
 } 
__p+='</div><a class="sub-title-w" href="'+
((__t=( data.author.link ))==null?'':__t)+
'"><div class="article-tags">';
 if (data.tags.length > 0) { 
__p+='';
 for (var tag in data.tags) { 
__p+='<div class="article-tag '+
((__t=( data.tags[tag] == '原创' ? 'original' : '' ))==null?'':__t)+
'">'+
((__t=( data.tags[tag] ))==null?'':__t)+
'</div>';
 } 
__p+='';
 } 
__p+='</div>';
 if (data.article.type === 'pgc') { 
__p+='<span class="sub-title">'+
((__t=( data.article.publishTime ))==null?'':__t)+
'</span>';
 } else if (data.article.type === 'forum') { 
__p+='<span class="sub-title">'+
((__t=( data.article.publishTime ))==null?'':__t)+
''+
((__t=( data.author.intro && data.article.publishTime ? '&nbsp;&middot;&nbsp;' : '' ))==null?'':__t)+
''+
((__t=( data.author.intro ))==null?'':__t)+
'</span>';
 } else { 
__p+='<span class="sub-title">'+
((__t=( data.author.intro ))==null?'':__t)+
'</span>';
 } 
__p+='</a></div></div>';
 /* 关注推荐容器 */ 
__p+='<div class="mediasug-outer-container"><div class="mediasug-inner-container"><div class="ms-pointer"></div><div class="ms-title">相关推荐</div><div class="ms-list" id="mediasug-list"><div class="ms-list-scroller" id="mediasug-list-html"></div></div></div></div>';
 /* 关注引导图片 */ 
__p+='<div class="concern-guide-picture" id="concern-guide-picture" show="'+
((__t=( (data.h5_settings.is_show_concern_guide_picture && data.author.followState !== 'following' && !data.author.isAuthorSelf) ? 'true' : 'false' ))==null?'':__t)+
'"></div>';
 /* copy作者区域for文章底部作者卡片 */ 
__p+='<div class="copy-authorbar" id="copy-authorbar" style="display: none;" topbutton-type="'+
((__t=( data.topbuttonType ))==null?'':__t)+
'"><div class="authorbar '+
((__t=( data.article.type ))==null?'':__t)+
'" id="profile-copy" data-href="'+
((__t=( data.author.link ))==null?'':__t)+
'">';
 /* 头像 */ 
__p+='<div class="author-avatar-link pgc-link"><img class="author-avatar" src="'+
((__t=( data.author.avatar ))==null?'':__t)+
'"></div>';
 /* 简易关注按钮 */ 
__p+='<div class="subscribe-button follow-button '+
((__t=( 'followState' in data.author ? data.author.followState : 'disabled' ))==null?'':__t)+
'"data-user-id="'+
((__t=( data.author.userId ))==null?'':__t)+
'"data-media-id="'+
((__t=( data.author.mediaId ))==null?'':__t)+
'"style="display: '+
((__t=( data.author.isAuthorSelf ? 'none' : 'block' ))==null?'':__t)+
';"data-concerntype="detail_card"><i class="iconfont focusicon">&nbsp;</i></div><div class="author-bar"><div class="name-link-w '+
((__t=( data.author.intro === '' ? 'no-intro' : '' ))==null?'':__t)+
'"><span class="author-name-link pgc-link">'+
((__t=( data.author.name ))==null?'':__t)+
'</span>';
 if (data.useServerV) { 
__p+='';
 if (data.author.auth_info) { 
__p+=''+
((__t=( buildServerVIcon(data.author.auth_type, 'label_icon') ))==null?'':__t)+
'';
 } 
__p+='';
 } else { 
__p+='';
 if (data.author.verifiedContent != '') { 
__p+='<div class="iconfont verified-icon">&#xe600;</div>';
 } 
__p+='';
 } 
__p+='</div><div class="sub-title-w"><span class="sub-title">'+
((__t=( data.author.intro ))==null?'':__t)+
'</span></div></div></div>';
 /* 作者菜单 */ 
__p+='';
 if (typeof data.menu === 'object' && data.menu.length > 0) { 
__p+='<div class="menu-container" style="display: none;">';
 for (var j = 0; j < data.menu.length; j++) { 
__p+='<div class="menu">';
 if ('children' in data.menu[j] && !('schema_href' in data.menu[j])) { 
__p+='<i class="iconfont menu-dot">'+
((__t=( '&#xe605;'))==null?'':__t)+
'</i>';
 } 
__p+=''+
((__t=( data.menu[j].name ))==null?'':__t)+
'</div>';
 } 
__p+='</div>';
 } 
__p+='</div>';
 } 
__p+='';
 /* PGC作者置顶测试 */ 
__p+='';
 if (data.h5_settings.pgc_over_head) { 
__p+='<div class="tt-title pgc-over-head">'+
((__t=( data.article.title ))==null?'':__t)+
'</div>';
 } 
__p+='</header>';
}
return __p;
};
var FooterTemplateFunction = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='';
 if (data.original) { 
__p+='<div class="carbon-copy"><span class="cc-text">转自头条号：</span><a class="cc-who" href="'+
((__t=( data.original.link ))==null?'':__t)+
'">'+
((__t=( data.original.name ))==null?'':__t)+
'</a></div>';
 } 
__p+='';
 if (data.wenda_extra) { 
__p+='';
 if (data.wenda_extra.wd_version >= 3) { 
__p+='<div class="wd-footer"><div class="publish-datetime">'+
((__t=( data.article.publishTime ))==null?'':__t)+
'</div>';
if(data.wenda_extra.wd_version >= 6){
__p+='<a class="report" style="display:none" onclick="ToutiaoJSBridge.call(\'report\')">举报</a><span style="display:none" class="sep for-report" style="font-size:12px;">|</span><a class="editor-edit-answer no-icon" style="display:none">编辑</a><div class="dislike-and-report no-icon" style="display:none;"  onclick="var that = this;ToutiaoJSBridge.call(\'dislike\', {options: 0x11},function(r){if(r.err_no==0){that.style.color=\'#999\';that.innerHTML=\'已反对\'}});">反对</div>';
}else{
__p+='<a class="editor-edit-answer" style="display:none">编辑</a><div class="dislike-and-report" onclick="ToutiaoJSBridge.call(\'dislike\', {options: 0x11});">不喜欢</div>';
}
__p+='</div>';
 } else { 
__p+='<div class="wenda-bottom clearfix"><div class="create-time">'+
((__t=( data.article.publishTime ))==null?'':__t)+
'</div></div><div class="bottom-buttons only-one"><div id="digg" data-answerid="'+
((__t=( data.wenda_extra.ansid ))==null?'':__t)+
'" class="ib like" wenda-state="" aniok="'+
((__t=( data.wenda_extra.aniok ))==null?'':__t)+
'"><span class="ibinner"><i class="iconfont iconb">&nbsp;</i><span class="b digg-count" realnum="0">赞</span></span></div><div id="bury" data-answerid="'+
((__t=( data.wenda_extra.ansid ))==null?'':__t)+
'" class="ib unlike" wenda-state="" aniok="'+
((__t=( data.wenda_extra.aniok ))==null?'':__t)+
'" style="display: none;"><span class="ibinner"><i class="iconfont iconb">&nbsp;</i><span class="b bury-count" realnum="0">踩</span></span></div></div>';
 } 
__p+='';
 } 
__p+='';
 if (data.novel_data) { 
__p+='<div class="serial">';
 if (data.novel_data.pre_group_url) { 
__p+='<a class="prev" id="prev_serial_link" href="'+
((__t=( data.novel_data.pre_group_url ))==null?'':__t)+
'">上一章</a>';
 } else { 
__p+='<span class="prev disabled">上一章</span>';
 } 
__p+='';
 if (data.novel_data.next_group_url) { 
__p+='<a class="next" id="next_serial_link" href="'+
((__t=( data.novel_data.next_group_url ))==null?'':__t)+
'">下一章</a>';
 } else { 
__p+='<span class="next disabled">下一章</span>';
 } 
__p+='<div class="index-wrap"><a class="index" id="index_serial_link" href="'+
((__t=( data.novel_data.url ))==null?'':__t)+
'">目录（共'+
((__t=( data.novel_data.serial_count ))==null?'':__t)+
'章）</a></div></div>';
 } else if (data.wenda_extra && data.wenda_extra.wd_version >= 1 && data.wenda_extra.wd_version < 3) { 
__p+='<div class="serial" style="display: '+
((__t=( data.wenda_extra.wd_version >= 2 ? 'block' : 'none' ))==null?'':__t)+
';"><a class="prev" id="wenda_index_link"><span id="total-answer-count-index"></span></a><a class="next" id="next_answer_link" onclick="ToutiaoJSBridge.call(\'tellClientRetryPrefetch\');">下一个回答</a></div>';
 } 
__p+='';
}
return __p;
};

(function() {
    window.globalArticleObject = mergeArguments();

    var headerTemplateString = HeaderTemplateFunction({
        data: globalArticleObject
    });

    window.headerTemplateString = headerTemplateString;

    var footerTemplateString = FooterTemplateFunction({
		data: globalArticleObject
	});

    document.body.classList.add(globalArticleObject.article.type);
    if (is_iphone()) {
        document.body.classList.add('iphone');
    } else if (is_android()) {
        document.body.classList.add('android');
    }
    document.body.classList.add('v55');

	// 做header，article，footer的合法检查
	// TODO 这里的问题是，没有顾及到三个dom的数量超过1时的情景
	var $header = $('header');
	var $article = $('article');
	var $footer = $('footer');

    if ($header.length <= 0) {
        $(document.body).prepend(headerTemplateString);
    } else {
        $header.replaceWith(headerTemplateString);
    }

    /**
     * 关于问答头部逻辑的描述
     * 旧版本问答头部由客户端实现，content中<.tt-title>内容为空
     * 某版本（wd_version = 1）后，问答标题改由前端展示。点击的逻辑是：从回答列表页进入，返回即可；从其他途径进入，跳转到列表页
     * 部分回答，由于服务端兼容性策略，导致<.tt-title>中有内容，故要在问答逻辑中，将其删掉
     * 某版本（wd_version = 3)后，尝试做ab测试，其中
     *  showMode = 1 版本，头部仍由客户端展示
     *  showMode != 1 版本，头部由web展示
     * 后，头部点击策略发生变化，由need_return标志位控制。
     * 后，头部添加回答入口，其跳转路径由服务端infor接口下发 FIXME
     * 后，头部回答入口的展示形式改为服务端控制。旧回答入口在未读取展示条件下可以不展示，以规避旧入口的数据处理时机问题。
     */
	if (globalArticleObject.article.type === 'wenda') {
		var ansStrategy = wenda_extra.show_post_answer_strategy || {};
		var wdVersion = wenda_extra.wd_version || 0;
		var isLiteApp = globalArticleObject.h5_settings.is_liteapp;
		var showBigAns = ('show_top' in ansStrategy) && !isLiteApp;
		var showSmlAns = ('show_default' in ansStrategy) && !isLiteApp;
		var $wdtitle;

		// 用于发送统计的参数集合
		var wendaStatisticsParams = {
			value: wenda_extra.qid,
			ext_value: wenda_extra.nice_ans_count,
			extra: {
				enter_from: wenda_extra.enter_from,
				ansid: wenda_extra.ansid,
				parent_enterfrom: wenda_extra.parent_enterfrom || ''
			}
		};

		// 预留一个回调用来设置放出回答数
		window.assignThroughWendaNiceanscount = function(nice_ans_count) {
			wendaStatisticsParams.ext_value = nice_ans_count;
		};

		if (wdVersion < 1 || (wdVersion >= 3 && wenda_extra.showMode == 1)) {
			$('header').find('.tt-title').remove();
		} else {
			if (showBigAns) {
				$wdtitle = $('<div class="wt">' + wenda_extra.title + '</div><div class="ft"><span class="see-all-answers" id="total-answer-count"></span><span class="hide-placeholder">&nbsp;</span></div><a class="big-answer-buttoon go-to-answer" href="' + ansStrategy.show_top.schema + '">' + ansStrategy.show_top.text + '</a><div class="big-answer-buttoon-gap"></div>');
			} else if (showSmlAns) {
				$wdtitle = $('<div class="wt">' + wenda_extra.title + '</div><div class="ft"><a class="go-to-answer go-to-answer-small" href="' + ansStrategy.show_default.schema + '">回答</a><span class="see-all-answers" id="total-answer-count"></span></div>');
			} else {
				$wdtitle = $('<div class="wt">' + wenda_extra.title + '</div><div class="ft"><span class="see-all-answers" id="total-answer-count"></span><span class="hide-placeholder">&nbsp;</span></div>');
			}

			var ansClass = showBigAns ? 'bigans' : (showSmlAns ? 'smlans' : 'noans');

			$('header').find('.tt-title').removeClass('tt-title').addClass('wenda-title ' + ansClass).html($wdtitle).on('click', function() {
				if ('need_return' in wenda_extra) {
					if (!!wenda_extra.need_return) {
						ToutiaoJSBridge && ToutiaoJSBridge.call('close');
					} else {
						window.location.href = wenda_extra.list_schema;
					}
				} else if (['click_answer', 'click_answer_fold'].indexOf(wenda_extra.enter_from) > -1) {
					ToutiaoJSBridge && ToutiaoJSBridge.call('close');
				} else {
					window.location.href = wenda_extra.list_schema;
				}
			});



			$.Press_State({
				bindSelector: '.wenda-title,.big-answer-buttoon',
				exceptSelector: '.go-to-answer-small,.see-all-answers',
				pressedClass: 'pressing',
				removeLatency: 500
			});

			// 只会有一个顶部回答按钮
			if (showBigAns) {
				send_umeng_event('answer_detail', 'top_write_answer_show', wendaStatisticsParams);
				$('.go-to-answer').on('click', function(ev) {
					ev.stopPropagation();
					send_umeng_event('answer_detail', 'top_write_answer', wendaStatisticsParams);
				});
			} else if (showSmlAns) {
				send_umeng_event('answer_detail', 'wirte_answer_show', wendaStatisticsParams);
				$('.go-to-answer').on('click', function(ev) {
					ev.stopPropagation();
					send_umeng_event('answer_detail', 'wirte_answer', wendaStatisticsParams);
				});
			}

			// 底部回答入口要在内容超过2屏时才展示
			// 前提是客户端支持回答schema
			if (('show_bottom' in ansStrategy) && $('article').height() > 2 * window.innerHeight && !isLiteApp) {
				var $btc = $('<a href="' + ansStrategy.show_bottom.schema + '" class="bottom-big-answer-button"><div class="pl wdi iconfont">&#xe62f;</div><div class="pr"><div class="wdq">' + wenda_extra.title + '</div><div class="wds">' + ansStrategy.show_bottom.text + '</div></div></a>');
				setTimeout(function() {
					$('footer').append($btc); // FIXME 时机
					$btc.on('click', function() {
						send_umeng_event('answer_detail', 'bottom_write_answer', wendaStatisticsParams);
					});
					send_exposure_event_once($btc.get(0), function() {
						send_umeng_event('answer_detail', 'bottom_write_answer_show', wendaStatisticsParams);
					}, true);
				}, 500);
			}
		}

		// 根据来源处理不同的点击“全部xx个回答”的逻辑
		$('#wenda_index_link').on('click', function(){
			if (['click_answer', 'click_answer_fold'].indexOf(wenda_extra.enter_from) > -1) {
				ToutiaoJSBridge && ToutiaoJSBridge.call('close');
			} else {
				window.location.href = wenda_extra.list_schema;
			}
		});
	}

    if (globalArticleObject.article.type === 'forum') {
        // 将帖子标题撤掉，放入正文一开始，同时标注文字内容段落？那没有标题又没有内容怎么办？
        var _$tarp = $('article p').eq(0);

        if (_$tarp.text() !== '') {
            _$tarp.addClass('first-forum-p');
        } else {
            _$tarp.before('<p class="first-forum-p">');
            _$tarp = _$tarp.prev();
        }

        if (globalArticleObject.article.title) {
            _$tarp.prepend('【' + globalArticleObject.article.title + '】');
        }
        if (globalArticleObject.forum_extra.forum_info) {
            _$tarp.prepend('<a href="' + globalArticleObject.forum_extra.forum_info.schema + '">#' + globalArticleObject.forum_extra.forum_info.name + '#</a>');
        }

        if (_$tarp.text () === '') {
            _$tarp.remove();
        }

        // 帖子将时间放在页面下，FIXME 危险：拼接字符串
        var _show = 'inline';
        if ($('.poi').length > 0) {
            $('.poi').each(function (idx, dom) {
                var locationText = dom.textContent;
                if ($(dom).closest('.tt-repost-thread').length > 0) {
                    // 加icon
                    $(dom).replaceWith('<p class="poi" style="overflow: hidden;"><span class="location">' + locationText + '</span></p>');
                    if ($('article>.poi').length === 0) {
                        // 加阅读数
                        $('article').append('<p class="poi" style="overflow: hidden;"><span style="display: ' + _show + ';">' + formatCount(null, forum_extra.read_count, '0') + '阅读</span></p>');
                    }
                } else {
                    // 加icon，加阅读数
                    $(dom).replaceWith('<p class="poi" style="overflow: hidden;"><span class="location">' + locationText + '</span><span style="display: ' + _show + ';">&thinsp;&thinsp;&middot;&thinsp;&thinsp;' + formatCount(null, forum_extra.read_count, '0') + '阅读</span></p>');
                }
            });
        } else {
            // 加阅读数
            $('article').append('<p class="poi" style="overflow: hidden;"><span style="display: ' + _show + ';">' + formatCount(null, forum_extra.read_count, '0') + '阅读</span></p>');
        }
    }

	if ($footer.length > 0) {
		$footer.html(footerTemplateString);
	} else if ($article.length > 0) {
		$footer = $('<footer>');
		$footer.html(footerTemplateString);
		$article.after($footer);
	}
})();

function setExtra(onlyH5extra) {
    if ('title' in onlyH5extra) { // iOS全量传递，android只传下发变量
        if (_.isEqual(onlyH5extra, window.h5_extra)) {
            return;
        }
        window.h5_extra = onlyH5extra;
    } else {
        window.h5_extra = $.extend(h5_extra, onlyH5extra);
    }

    window.globalArticleObject = mergeArguments();

    // rerender just header
    var headerTemplateString = HeaderTemplateFunction({
        data: globalArticleObject
    });
    $('header').replaceWith(headerTemplateString);
    bindStatisticsEvents2(); // FIXME 改成body代理

    // NOTE
    // 1. meida_user_id 写在了h5_extra而非media
    // 2. 底部卡片的开关在context下发，要注意时序，并且header的重新渲染一定要在开关之前
}
;

/**
 * 引入客户端接口文件
 */
/****************************************************************
 * 这里面的方法都是供客户端调用的
 *****************************************************************/
/**
 * 帖子调用更新标签信息
 * @param  {array|string} tags 要展示的tags
 * @return {undefined}
 */
function update_forum_tags (tags) {
    if (typeof tags === 'string') {
        tags = tags.split(',');
    }
    var $newtag = $('<div class="article-tags">');
    tags.forEach(function(tag){
        if (tag !== '') {
            $newtag.append($('<div class="article-tag">').html(tag));
        }
    });

    // 处理姓名单行居中问题
    if (tags.length >= 1) {
        $('.name-link-w').removeClass('no-intro');
    } else if ($('.sub-title').text() === '') {
        $('.name-link-w').addClass('no-intro');
    }

    $('.article-tags').replaceWith($newtag);
}

/**
 * 客户端在webview“挂起”时调用
 * @return {undefined}
 */
function on_page_disappear () {
    // 停止所有音乐的播放
    // NOTE 应PM要求暂时去掉
    // $('audio').each(function(){
    //     if (!this.paused) {
    //         sendAudioEvent.call(this, 'sound_over', 'quit_detail');
    //         this.pause();
    //     }
    // });
    mediasugScroll.pushimpr();
}

/**
 * 问答、帖子调用更新信息
 * NOTE
 * 问答从『某版本』后不再调用此接口，转而使用wenda_context
 * 帖子从『某版本』后，非作者本人时，如已关注也不再显示关注按钮
 * @param {object} states 透传信息
 */
var globalWendaStates = {};
function set_info (states) {
    if (typeof states === 'string') {
        states = JSON.parse(states); // android貌似不能传递对象
    } else if (typeof states !== 'object') {
        return;
    }

    $.extend(window.globalWendaStates, states); // TODO 改成debug模式才写全局变量

    // 关注状态
    if ('is_concern_user' in states) {
        change_following_state(!!states.is_concern_user);
    }

    // // 作者本人不展示关注按钮
    // if ('isAuthor' in states) {
    //     if (states.isAuthor) {
    //         $('.follow-button').hide();
    //         $('.author-function-buttons').hide();
    //         $('.wenda-info').show();
    //     } else {
    //         $('.author-function-buttons').show();
    //         $('.follow-button').show();
    //         $('.wenda-info').hide();
    //     }
    // }

    // 问答数据
    // 浏览
    if ('brow_count' in states) {
        $('.brow-count').text(states.brow_count);
        formatCount('.brow-count', states.brow_count, '0');
    }

    // 点赞
    if ('is_digg' in states && 'digg_count' in states) {
        if (states.is_digg) {
            $('#digg').attr({
                'wenda-state': 'digged',
                'aniok': 'false'
            });
        }
        formatCount('.digg-count', states.digg_count, '赞');
        formatCount('.digg-count-special', states.digg_count, '0');
    }

    // 踩
    if ('is_buryed' in states && 'bury_count' in states) {
        if (states.is_buryed) {
            $('#bury').attr({
                'wenda-state': 'buryed',
                'aniok': 'false'
            });
        }
        formatCount('.bury-count', states.bury_count, '踩');
    }

    // 踩按钮开关
    if ('is_show_bury' in states && states.is_show_bury) {
        $('#bury').show().parent().removeClass('only-one').addClass('only-two');
    }
}

/**
 * 获取header的位置
 * @return {undefined}
 * @ios
 * NOTE 为解决iOS的UIWebview上scroll事件不停手就不执行逻辑的bug
 */
function getElementPosition (selector) {
    var dom = document.querySelector(selector);
    if (dom) {
        var coords = dom.getBoundingClientRect();
        return '{{' + coords.left + ',' + dom.offsetTop + '},{' + coords.width + ',' + coords.height + '}}';
    }
    return '{{0,0},{0,0}}';
}

/**
 * 设置正文字号大小
 * @param {string} s 类型[_标准正文字号]
 * NOTE px先保留，后续处理
 */
function setFontSize (s) {
    var type = s.split('_')[0];
    var px = s.split('_')[1];
    var validTypes = ['s', 'm', 'l', 'xl'];
    var allClasses = $.map(validTypes, function(i){
        return 'font_' + i;
    }).join(' ');

    if (validTypes.indexOf(type) > -1) {
        $('body').removeClass(allClasses).addClass('font_' + type);
    }
}

/**
 * 设置日夜间模式
 * @param {number} flag 1代表日间，0代表夜间
 */
function setDayMode (flag) {
    var validFlags = [0, 1, '0', '1'];
    if (validFlags.indexOf(flag) > -1) {
        flag = parseInt(flag);
        $('body')[flag ? 'removeClass' : 'addClass']('night');
    }
}

var TouTiao = {
    setFontSize: setFontSize,
    setDayMode: setDayMode
};

/**
 * 关闭视频时，APP调用此函数通知web，恢复视频原始位置
 * @param {number} vid 视频ID
 * @android
 */
function appCloseVideoNoticeWeb (vid) {
    var $video = $('[data-vid="' + vid + '"]');
    $video.each(function(idx, video){
        $(this).css('display', 'block');
        $(this).next('.cv-wd-info-wrapper').show();
        $('body').css('margin-top', '0px');
    });
}

/**
 * 获取vid对应的视频的位置、尺寸信息供客户端横屏-->竖屏变换时恢复视频使用
 * @param {number} vid 视频id
 * @return {string} 尺寸
 * @ios
 */
function getVideoFrame (vid) {
    var video = document.querySelector('[data-vid="' + vid + '"]');
    var frame = '{{0,0},{0,0}}';
    if (video) {
        var coords = video.getBoundingClientRect();
        frame = '{{' + coords.left + ',' + video.offsetTop + '},{' + coords.width + ',' + coords.height + '}}';
    }
    return frame;
}

/**
 * 举报错别字的接口回调
 * 将选中文字前后20个字（如果有）回传给客户端
 */
function getThreeStrings() {
    var one = '', two = '', three = '';

    var sel = document.getSelection();
    if (sel.type !== 'Range') {
        return [one, two, three];
    }

    var range = sel.getRangeAt(0);
    if (!range) {
        return [one, two, three];
    }

    try {
        one = range.startContainer.textContent.substring(0, range.startOffset).substr(-20);
        two = range.toString();
        three = range.endContainer.textContent.substring(range.endOffset).substring(0, 20);
    } catch(ex) {
        // 防止连续调用出现问题
    }

    range.detach();
    range = null;
    return [one, two, three];
}
;

/**
 * 引入服务端接口文件
 */
/****************************************************************
 * 这里的方法都是供服务端调用的
 *****************************************************************/
/**
 * @deprecated
 * 赞赏历史人数实时更新
 * @param  {number} latest_praise_num 最新赞赏人数
 * @return {undefined}
 */
function updateAppreciateCountByServer (latest_praise_num) {}

/**
 * 服务端告知是否已订阅头条号
 * @param  {boolean} following 是否已订阅
 * @param  {string}  host       访问订阅API的host
 * @param  {string}  args       访问订阅API时需要携带的其他参数
 * @return {undefined}
 * NOTE 此逻辑应只在pgc页面生效
 */
function subscribe_switch (following, host, args) {
    if (globalArticleObject.article.type == 'pgc') {
        change_following_state(!!following);
    }
}

/**
 * 后期追加页面信息，比如用于A/B测试
 * @param {object} context 服务端下发数据
 */
window.infoInserted = false;
function insertDiv (context) {
    if (!window.infoInserted) {
        try {
            /**
 * 根据服务端information接口context字段中的JSON格式数据，由前端构造相应DOM插入到页面中
 *
 * 统一的交由客户端调用的接口签名是 insertDiv(JSON data);
 *
 * NOTE
 * 1. 由于information为异步加载，输出内容如果在页面首屏，效果会非常差。
 */
var contextRenderer = function(context){

	if (typeof context !== 'object') {
		return;
	}

	window.globalContext = context;

	window.on_card_image_load_success = function(dom) {
		dom.style.opacity = 1;
	};

	/**
	 * 某版本后，问答额外信息由服务端下发
	 */
	(function () {
		if (!('wenda_context' in context)) {
			return;
		}

		var states = context.wenda_context;

		// 作者本人不展示关注按钮
	    if ('is_author' in states) {
	        if (states.is_author) {
	            $('.follow-button').hide();
	            $('.author-function-buttons').hide();
	            $('.wenda-info').show();
	        } else if (globalArticleObject.article.type === 'wenda' && globalArticleObject.h5_settings.is_liteapp) {
	            $('.follow-button').hide();
	            $('.author-function-buttons').hide();
	            $('.wenda-info').hide();
	        } else {
	            $('.author-function-buttons').show();
	            $('.follow-button').show();
	            $('.wenda-info').hide();
	        }
	    }


		// 问答部分    作者本人 右下角不喜欢改成编辑
		// console.log('#####################');
		// console.log(context);
		// window.context = context;
	    if ('is_author' in states && states.is_author) {
			$('.wd-footer .editor-edit-answer').attr('href',states.edit_answer_schema).show();
			$('.wd-footer .dislike-and-report').hide();
		}else{
			$('.wd-footer .editor-edit-answer').hide();
			$('.wd-footer .dislike-and-report').show();
		}

		// if ('is_show_bury' in states && states.is_show_bury) {
		// 	// 展示反对
		// 	$('.wd-footer .dislike-and-report').show();
		// }
		
		// if ('is_show_edit' in states && states.is_show_edit){
		// 	// 展示编辑
		// 	if(states.edit_answer_schema){
		// 		$('.wd-footer .editor-edit-answer').attr('href',states.edit_answer_schema).show();
		// 	}
		// }


		// if('is_show_report' in states && states.is_show_report){
		if(1){
			// 显示举报
			$('.report').show();
			$('.sep.for-report').show();
		}


	    // 问答数据
	    // 浏览
	    if ('brow_count' in states) {
	        $('.brow-count').text(states.brow_count);
	        formatCount('.brow-count', states.brow_count, '0');
	    }

	    // 点赞
	    if ('is_digg' in states && 'digg_count' in states) {
	        if (states.is_digg) {
	            $('#digg').attr({
	                'wenda-state': 'digged',
	                'aniok': 'false'
	            });
	        }
	        formatCount('.digg-count', states.digg_count, '赞');
	        formatCount('.digg-count-special', states.digg_count, '0');
	    }

	    // 踩
	    if ('is_buryed' in states && 'bury_count' in states) {
	        if (states.is_buryed) {
	            $('#bury').attr({
	                'wenda-state': 'buryed',
	                'aniok': 'false'
	            });
	        }
	        formatCount('.bury-count', states.bury_count, '踩');
			if(states.is_buryed){
				$('.dislike-and-report').css("color","#999999").text('已反对');
			}
	    }

	    // 踩按钮开关
	    if ('is_show_bury' in states && states.is_show_bury) {
	        $('#bury').show().parent().removeClass('only-one').addClass('only-two');
	    }

	    // 关注状态
	    if ('is_concern_user' in states) {
	        change_following_state(!!states.is_concern_user);
	    }

		// NOTE 新版问答相关逻辑
		//    1. 顶部标题栏由web实现，回答总数在context传递
		//    2. 底部增加目录功能，是否展示由show_next确定，其中：
		//     2.1 下一页是否存在，由has_next确定，跳转地址由next_answer_schema指定
		//     2.2 全部的动作由访问路径决定，非click_answer都跳转到list_schema
		// UPDATE
		//    1. wenda_extra.wd_version >= 2 开始，永久显示底部目录
		if ('show_next' in states && !states.show_next) {
			return;
		} else {
			$('.serial').show();
		}

		// 问答下一页
		if ('has_next' in states) {
	        var $next = $('#next_answer_link');
	        if (states.has_next) {
	            $next.attr('href', states.next_answer_schema);
				$next.attr('onclick', null);
	        } else {
				$next.attr('onclick', null);
	            $next.addClass('disabled').on('click', function(){
	                ToutiaoJSBridge.call('toast', {
	                    text: '这是最后一个回答',
	                    icon_type: ''
	                });
	            });
	        }
	    }

	    // 回填问答标题和目录上的全部回答数
	    if ('ans_count' in states) {
	        $('#total-answer-count').html(states.ans_count + '个回答').css('display', 'inline-block');
	        $('#total-answer-count-index').html('全部' + states.ans_count + '个回答');
	    }

		// 当获取到放出回答数时，通过前置的回调方法设置wenda_extra变量值，或直接赋值
		if ('nice_ans_count' in states && 'wenda_extra' in window) {
			if (typeof window.assignThroughWendaNiceanscount === 'function') {
				window.assignThroughWendaNiceanscount(states.nice_ans_count);
			} else {
				window.wenda_extra.nice_ans_count = states.nice_ans_count;
			}
		}
	    //*/
	})();

    (function() {
        if ('wenda_extra' in window && wenda_extra.wd_version >= 3 && wenda_extra.showMode == 1) {
            if (typeof context.wenda_context !== 'object') {
                return;
            }
            if (!('comment_list' in context.wenda_context) || !('ans_count' in context.wenda_context)) {
                return;
            }

            var createWendaCommentsTemplate = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='';
 for (var i = 0; i < data.comment_list.length; i++) { var _tempComment = data.comment_list[i] 
__p+='<li class="wd-comment" data-comment-id="'+
((__t=( _tempComment.comment_id ))==null?'':__t)+
'"><div class="header"><a class="avatar-container" href="'+
((__t=( _tempComment.user_info.schema ))==null?'':__t)+
'"><img class="avatar" src="'+
((__t=( _tempComment.user_info.avatar_url ))==null?'':__t)+
'"></a><div class="admire iconfont" '+
((__t=( !!_tempComment.is_digg ? 'digged' : '' ))==null?'':__t)+
' data-comment-id="'+
((__t=( _tempComment.comment_id ))==null?'':__t)+
'" realnum="'+
((__t=( _tempComment.digg_count ))==null?'':__t)+
'">'+
((__t=( formatCount(null, _tempComment.digg_count, '赞') ))==null?'':__t)+
'</div><div class="profile"><a class="name" href="'+
((__t=( _tempComment.user_info.schema ))==null?'':__t)+
'">'+
((__t=( _tempComment.user_info.uname ))==null?'':__t)+
'';
 if (_tempComment.is_verify) { 
__p+='<span class="iconfont vip-icon">&#xe600;</span>';
 } 
__p+='</a></div></div><div class="container"><div class="content">'+
((__t=( _tempComment.content ))==null?'':__t)+
'<!-- ';
 if ('reply_to_comment' in _tempComment) { 
__p+='//<span class="quote-name">'+
((__t=( _tempComment.reply_to_comment.user_name ))==null?'':__t)+
'</span><span class="quote-content">'+
((__t=( _tempComment.reply_to_comment.text ))==null?'':__t)+
'</span>';
 } 
__p+=' --></div><div class="function"><div class="datetime">'+
((__t=( commentTimeFormat(_tempComment.create_time) ))==null?'':__t)+
'</div></div></div></li>';
 } 
__p+='';
 if (data.totalCommentsCount > data.comment_list.length && data.comment_list.length > 0) { 
__p+='<div class="check-all-comments" onclick="ToutiaoJSBridge.call(\'openComment\');">查看全部'+
((__t=( data.totalCommentsCount ))==null?'':__t)+
'条回复</div>';
 } 
__p+='';
}
return __p;
};
            var $commentsTpl = $(createWendaCommentsTemplate({
                data: context.wenda_context
            }));

            var $commentsContainer = $('<ul class="wd-comments"></ul>');
            $commentsContainer.append($commentsTpl);
            $('footer').append($commentsContainer);

            $commentsContainer.on('click', '.admire', function(ev) {
                // ev.preventDefault();
                // ev.stopPropagation();
                // console.info('======');
                var admireState = this.getAttribute('digged');
                var commentId = this.dataset.commentId;
                var commentCount = +this.getAttribute('realnum');
                if (admireState === null) {
                    this.setAttribute('digged', true);
                    formatCount(this, commentCount+1, '赞');
                    ToutiaoJSBridge.call('commentDigg', {
                        commentId: commentId,
                        action: 'digged'
                    });
                } else {
                    ToutiaoJSBridge.call('toast', {
                        text: '您已经赞过'
                    });
                }
            }).on('click', '.wd-comment', function(ev) {
                // ev.preventDefault();
                if ($(ev.target).closest('.header').length > 0) {
                    return;
                }
                var commentId = this.dataset.commentId;
                ToutiaoJSBridge.call('openComment', {
                    commentId: commentId
                });
                // location.href = 'sslocal://webview?url=xx'
            });
        }
    })();

	/**
	 * 详情页提问功能
	 */
	(function () {
		if (context.is_question_shown != 1 || globalArticleObject.article.type !== 'pgc') {
			return;
		}
		var createArticleAskTemplate = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<a class="cardx ask" href="'+
((__t=( data.url ))==null?'':__t)+
'" style="visibility: hidden;"><div class="title" id="cutwrapper"><i class="iconfont ttwdlogo">&#xe633;</i>关于「<span id="cut">'+
((__t=( data.title ))==null?'':__t)+
'</span>」，提个问题跟大家讨论下<i class="iconfont rightarrowicon">&#xe644;</i></div></a>';
}
return __p;
};
		var $articleAskTpl = $(createArticleAskTemplate({
			data: {
				url: 'sslocal://wenda_question_post?api_param=' + encodeURIComponent(JSON.stringify({
					group_id: h5_extra.str_group_id,
					item_id: h5_extra.str_item_id,
					enter_from: 'group_detail'
				})),
				title: globalArticleObject.article.title
			}
		}));

		$articleAskTpl.on('click', function () {
			send_umeng_event('ask_question', 'article_ask_click', {
				value: h5_extra.str_group_id,
				extra: {
					item_id: h5_extra.str_item_id
				}
			});
		});

		$('footer').append($articleAskTpl);

		var height = $('#cutwrapper').height();
		if (height > 27*3) {
			do {
				$('#cut').html($('#cut').html().slice(0, -3));
				height = $('#cutwrapper').height();
			} while (height > 27*3)
			$('#cut').html($('#cut').html().slice(0, -3) + '...');
		}
		$articleAskTpl.css('visibility', 'visible');

		send_exposure_event_once($articleAskTpl.get(0), function () {
			send_umeng_event('ask_question', 'article_ask_show', {
				value: h5_extra.str_group_id,
				extra: {
					item_id: h5_extra.str_item_id
				}
			});
		}, true);
	})();

	/**
	 * 卡片
	 */
	(function(){
		var LiveTalkTemplate = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<a class="newcard-wrapper live '+
((__t=( type ))==null?'':_.escape(__t))+
'" href="'+
((__t=( url ))==null?'':_.escape(__t))+
'"><div class="newcard-inner"><div class="newcard-left"><div class="iconfont playicon"></div><img class="pimage" src="'+
((__t=( icon ))==null?'':_.escape(__t))+
'" /></div><div class="newcard-right"><h2 class="title">'+
((__t=( title ))==null?'':_.escape(__t))+
'</h2><div class="btms"><div class="btms-left"><div class="live-status" status-code="'+
((__t=( status ))==null?'':_.escape(__t))+
'"><i class="iconfont statusicon">&nbsp;</i>'+
((__t=( status_display ))==null?'':_.escape(__t))+
'</div><div class="sub-title">'+
((__t=( sub_title ))==null?'':_.escape(__t))+
'</div></div></div></div></div></a>';
}
return __p;
};
		var cardTemplateFunctions = {
			'movie': function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<a class="newcard-wrapper movie" href="'+
((__t=( url ))==null?'':_.escape(__t))+
'"><div class="newcard-inner"><div class="newcard-left"><div class="iconfont playicon"></div><img class="pimage" src="'+
((__t=( poster ))==null?'':_.escape(__t))+
'" /></div><div class="newcard-right"><h2 class="title">'+
((__t=( name ))==null?'':_.escape(__t))+
'';
 if (tag !== '') { 
__p+='<span class="rytag">'+
((__t=( tag ))==null?'':_.escape(__t))+
'</span>';
 } 
__p+='</h2><div class="btms"><div class="btms-right"><button class="button">进入</button></div><div class="btms-left">';
 if (typeof actor_name === 'string') { 
__p+='<p class="sub-title actor">'+
((__t=( actor_name ))==null?'':__t)+
'</p>';
 } 
__p+='<p class="sub-title">'+
((__t=( desc ))==null?'':_.escape(__t))+
'</p><p class="sub-title">'+
((__t=( sub_title ))==null?'':_.escape(__t))+
'</p>';
 if (typeof score === 'number') { 
__p+='<p class="sub-title last-title"><span class="iconfont film-star-score" data-score="'+
((__t=( Math.ceil(score) ))==null?'':_.escape(__t))+
'">'+
((__t=( score.toFixed(1) ))==null?'':_.escape(__t))+
'</span></p>';
 } else { 
__p+='<p class="sub-title last-title nostar">暂无评分</p>';
 } 
__p+='</div></div></div></div></a>';
}
return __p;
},
			'live_talk_star': LiveTalkTemplate,
			'live_talk_match': LiveTalkTemplate,
			'live_talk_video': LiveTalkTemplate,
			'fiction': function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<a class="card fiction" href="'+
((__t=(url))==null?'':_.escape(__t))+
'"><div class="card-hd"><img class="img" src="'+
((__t=(poster))==null?'':_.escape(__t))+
'"></div><div class="card-bd"><p class="title">'+
((__t=(name))==null?'':_.escape(__t))+
'</p><p class="sub-title">'+
((__t=(abstract))==null?'':_.escape(__t))+
'</p><p class="sub-title"><span class="categ">'+
((__t=(category))==null?'':_.escape(__t))+
'</span>'+
((__t=(note))==null?'':_.escape(__t))+
'</p></div><div class="card-ft"><button class="button" action="concern" is-concerned="'+
((__t=(Boolean(is_concerned)))==null?'':_.escape(__t))+
'" concern-id="'+
((__t=(concern_id))==null?'':_.escape(__t))+
'" forum-id="'+
((__t=(forum_id))==null?'':_.escape(__t))+
'">'+
((__t=( is_concerned ? '已关注' : '关注' ))==null?'':_.escape(__t))+
'</button></div></a>';
}
return __p;
},
			'auto': function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<div class="pcard"><div class="pcard-caption"><span class="pcard-h14 pcard-w1">相关车型</span><span class="pcard-h14 pcard-w3 pcard-vr" style="width: 16px;"></span><span class="pcard-h14 pcard-w3">数据来源于易车</span></div><div class="p-autocard pcard-container pcard-vertical-border" data-href="'+
((__t=( data.open_url ))==null?'':__t)+
'" data-content="content"><div class="pcard-clearfix"><div class="auto-image pcard-pull-left" style="background-image: url('+
((__t=( data.cover_url ))==null?'':__t)+
');"></div><div class="" style="margin-left: 100px;"><button type="button" class="pcard-button pcard-pull-right" data-href="'+
((__t=( data.open_url ))==null?'':__t)+
'">详情</button><div class="" style="margin-right: 88px;"><div class="pcard-h16 pcard-w1" style="margin-bottom: 5px;">'+
((__t=( data.car_series))==null?'':__t)+
'</div><div class="pcard-h12 iconfont film-star-score" data-score="'+
((__t=( Math.ceil(data.rate*2) ))==null?'':__t)+
'" style="margin-bottom: 5px;">'+
((__t=( data.rate ))==null?'':__t)+
'</div></div><div class="pcard-h14 pcard-w1">参考成交价: <span class="pcard-w4">￥'+
((__t=( data.price ))==null?'':__t)+
'</span></div></div></div><div class="" style="margin-top: 8px;"><a class="pcard-h14 pcard-w1-a" href="'+
((__t=( data.jump_url[0] ))==null?'':__t)+
'" data-label="card_ask">询底价</a><span class="pcard-h14 pcard-w3 pcard-vr" style="width: 30px;"></span><a class="pcard-h14 pcard-w1-a" href="'+
((__t=( data.jump_url[1] ))==null?'':__t)+
'" data-label="card_second">二手车</a><span class="pcard-h14 pcard-w3 pcard-vr" style="width: 30px;"></span><a class="pcard-h14 pcard-w1-a" href="'+
((__t=( data.jump_url[2] ))==null?'':__t)+
'" data-label="card_sale">厂商活动</a><span class="pcard-h14 pcard-w3 pcard-vr" style="width: 30px;"></span><a class="pcard-h14 pcard-w1-a" href="'+
((__t=( data.jump_url[3] ))==null?'':__t)+
'" data-label="card_stages">分期买车</a></div></div></div>';
}
return __p;
}
		};

		if (!('cards' in context) || !Array.isArray(context.cards)) {
			return;
		}

		context.cards.forEach(function (card) {
			var cardType = card.type;
			var statisticsObject = {
				value: globalArticleObject.statistics.group_id,
				extra: {
					item_id: globalArticleObject.statistics.item_id,
					card_type: card.type,
					card_id: card.id
				}
			};

			if (!(cardType in cardTemplateFunctions)) {
				return;
			}
            if (cardType === 'auto') {
                if (!Array.isArray(card.jump_url) || card.jump_url.length == 0) {
                    return;
                }
                var $template = $(cardTemplateFunctions[cardType]({
                    data: card
                }));
            } else {
	            var $template = $(cardTemplateFunctions[cardType](card));
            }

			$template.find('.button').on('click', function (ev) {
				ev.stopPropagation();
				send_umeng_event('detail', 'click_card_button', statisticsObject);

				var $button = $(this);
				// 目前只有小说的卡片有这个按钮
				// 这里如何拆解看起来比较不能尽如人意
				if ($button.attr('action') === 'concern') {
					ev.preventDefault();

					if (card.isclicking) {
						return;
					}
					card.isclicking = true;

					send_umeng_event('detail', card.is_concerned ? 'click_fictioncard_discare' : 'click_fictioncard_care', statisticsObject);

					var xhrp = $.ajax({
						url: 'http://ic.snssdk.com/concern/v1/commit/' + (card.is_concerned ? 'discare/' : 'care/'),
						dataType: 'jsonp',
						data: {
							concern_id: card.concern_id
						},
						success: function(data){
							card.isclicking = false;
							if (data.err_no == 0) {
								card.is_concerned = !card.is_concerned;
								$button.attr('is-concerned', Boolean(card.is_concerned))
									.html(card.is_concerned  ? '已关注' : '关注');
								// 成功之后通知其他页面，为兼容话题版本需要关心话题都发
								ToutiaoJSBridge.call('page_state_change', {
									type: 'concern_action',
									id: card.concern_id,
									status: card.is_concerned ? 1 : 0
								});
								ToutiaoJSBridge.call('page_state_change', {
									type: 'forum_action',
									id: card.forum_id,
									status: card.is_concerned ? 1 : 0
								});
								send_umeng_event(card.is_concerned ? 'concern_novel' : 'unconcern_novel', 'detail', {
									value: h5_extra.str_group_id,
									extra: {
										item_id: h5_extra.str_item_id,
										novel_id: card.id
									}
								});
							} else {
								ToutiaoJSBridge.call('toast', {
									text: '操作失败，请重试',
									con_type: 'icon_error'
								});
							}
						},
						error: function(){
							card.isclicking = false;
							ToutiaoJSBridge.call('toast', {
								text: '操作失败，请重试',
								icon_type: 'icon_error'
							});
						}
					});
				}
			});

			if (cardType === 'auto') {
                $template.find('[data-label]').on('click', function (ev) {
                    ev.stopPropagation();
                    send_umeng_event('detail', 'click_' + this.dataset.label, statisticsObject);
                });
                $template.find('[type="button"]').on('click', function (ev) {
                    ev.stopPropagation();
                    location.href = this.dataset.href;
                    send_umeng_event('detail', 'click_card_button', statisticsObject);
                });
                $template.on('click', '[data-content]', function (ev) {
                    location.href = this.dataset.href;
                    send_umeng_event('detail', 'click_card_content', statisticsObject);
                });
                $.Press_State({
                    bindSelector: '.p-autocard',
                    exceptSelector: '[data-label],[type="button"]',
                    removeLatency: 500
                });
            } else {
                $template.on('click', function (ev) {
                    send_umeng_event('detail', 'click_card_content', statisticsObject);
                });
            }

			$('footer').prepend($template);

            setTimeout(function(){
                console.info($template.get(0));
            }, 10000);
			send_exposure_event_once($template.get(0), function () {
				send_umeng_event('detail', 'card_show', statisticsObject);
			}, true);
		});
	})();

	/**
	 * 问答导流
	 * TODO 这里没有统计?
	 */
	(function(){
		if ('wenda_recommend' in context && 'wenda_extra' in window) {
			var templateFunction = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<a class="jrwdpd" href="'+
((__t=( data.open_url ))==null?'':__t)+
'"><div class="jrwdpd-slogan">'+
((__t=( data.text ))==null?'':__t)+
'</div><button class="jrwdpd-button">'+
((__t=( data.button_text ))==null?'':__t)+
'</button><div class="jrwdpd-logo"></div><div class="jrwdpd-logo-wrap"></div></a>';
}
return __p;
};
			var $template = $(templateFunction({
				data: context.wenda_recommend
			}));
			$('footer').append($template);

            $template.on('click', function() {
                send_umeng_event('wenda_channel_detail', 'enter', {
                    extra: {
                        qid: wenda_extra.qid,
                        ansid: wenda_extra.ansid,
                        enter_from: wenda_extra.enter_from,
                        parent_enterfrom: wenda_extra.parent_enterfrom || ''
                    }
                });
            });
		}
	})();

	/**
	 * 图集平铺测试中相关图集内容
	 */
	(function () {
	    if ('related_gallery' in context && Array.isArray(context.related_gallery)) {

	        var templateFunction = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<div class="related-gallery-topgap">&nbsp;</div><div class="related-gallery-container"><div class="related-gallery-title">相关图集</div><div class="related-gallery-list">';
 for (var i = 0; i < related_gallery.length; i++) { 
__p+='<div class="related-gallery-item" data-href="'+
((__t=( related_gallery[i]['open_url'] ))==null?'':_.escape(__t))+
'" data-show="'+
((__t=( related_gallery_section > i ? 'yes' : 'no' ))==null?'':_.escape(__t))+
'"><div class="image-placeholder" style="background-image: url('+
((__t=( related_gallery[i]['middle_image']['url'] ))==null?'':_.escape(__t))+
');"><img style="display: none;" src="'+
((__t=( related_gallery[i]['middle_image']['url'] ))==null?'':_.escape(__t))+
'" /><div class="subscript">'+
((__t=( related_gallery[i]['image_count'] || '多' ))==null?'':_.escape(__t))+
'图</div></div><div class="desc-wrapper"><div class="desc">'+
((__t=( related_gallery[i]['title'] ))==null?'':_.escape(__t))+
'</div></div></div>';
 } 
__p+='</div>';
 if (related_gallery_section < i) { 
__p+='<div class="seemore">查看更多<i class="iconfont seemoreicon">&#xe627;</i></div>';
 } 
__p+='</div>';
}
return __p;
};
	        var $template = $(templateFunction(context));

	        $template.on('click', '.seemore', function (ev) {
	            $(this).remove();
	            $template.attr('show-all', '');
				try {
					send_umeng_event('detail', 'click_related_gallery_more', {
		                value: h5_extra.media.id, // FIXME
		                extra: {
		                    item_id: h5_extra.item_id
		                }
		            });
				} catch (err) {}
	        }).on('click', '.related-gallery-item', function (ev) {
	            var href = $(this).data('href');
	            location.href = href;
	        });

	        $('footer').append($template);
	    }
	})();

    /**
     * pgc底部作者信息区域
     * 从进页面处理移动到此处
     * 如果中了AB测，不管有没有菜单数据都可以展示copy-authorbar
     * 如果内容不足一屏幕，不应当显示出来
     */
    (function() {
        if (!!context.is_show_author_card && globalArticleObject.article.type === 'pgc' && $('article').height() > window.innerHeight) {
            var $copy = $('#copy-authorbar');
            var statisticsObject = {
                value: h5_extra.str_group_id,
                extra: {
                    item_id: h5_extra.str_item_id,
                    card_type: globalArticleObject.menu ? 'pgc_author_card_menu' : 'pgc_author_card',
                    card_id: globalArticleObject.author.userId,
                    card_mid: globalArticleObject.author.mediaId
                }
            };

            $copy.find('.authorbar').on('click', function(ev) {
                window.location.href = $(this).attr('data-href');
                send_umeng_event('detail', 'click_card_content', statisticsObject);
            });

            $copy.find('.subscribe-button').on('click', function(ev) {
                ev.stopPropagation();
                ev.preventDefault();
                send_umeng_event('detail', 'click_card_button', $.extend(true, {}, statisticsObject, {
                    extra: {
                        status: $(this).hasClass('following') ? 1 : 0
                    }
                }));
            });

            $copy.find('.menu').on('click', function(ev) {
                ev.stopPropagation();
                var _index = $(this).index();
                var _menu = globalArticleObject.menu[_index];
                var _statisticsObject = $.extend(true, {}, statisticsObject);
                _statisticsObject.extra.firstmenu_id = _index;
                send_umeng_event('detail', 'click_card_firstmenu', _statisticsObject);
                if (_menu.children.length === 0 && _menu.schema_href) {
                    // 只有一级菜单，直接跳转
                    window.location.href = _menu.schema_href;
                } else if (_menu.children.length > 0) {
                    // 二级菜单，调用jsBridge通知客户端
                    _menu.firstmenu_id = _index;
                    ToutiaoJSBridge.call('showActionSheet', _menu, null);
                }
            });

            $.Press_State({
                bindSelector: '.authorbar',
                exceptSelector: '.subscribe-button',
                pressedClass: 'pressing',
                removeLatency: 500
            });

            $copy.prependTo('footer').show();

            send_exposure_event_once($copy.get(0), function() {
                send_umeng_event('detail', 'card_show', statisticsObject);
            }, true);
        }
    })();

	/**
	 * 了解更多
	 */
	(function() {
		if ('know_more_url' in context) {
			$('article').append('<p><a href="sslocal://webview?url=' + encodeURIComponent(context.know_more_url) + '&title=' + encodeURIComponent('网页浏览') + '">了解更多</a></p>');
		}
	})();
};
;
            contextRenderer(context);
            insertDivCallback();
        } catch (err) {
            window.insertDivError = err;
        } finally {
            window.infoInserted = true;
        }
    }
}

/**
 * 视频自动播放功能（在首屏或者滚动到屏幕中的第1个视频自动播放，后续视频忽略）
 * @return {undefined}
 */
window.autoplayed = false; //页面第一个视频是否已经自动播放
function videoAutoPlay () {
    var $videos = $('.custom-video');
	if (!autoplayed && $videos.length) {
		var firstVideo = $videos.get(0);
		if (_videoInView(firstVideo)) {
			playVideo(firstVideo, 1);
			autoplayed = true;
		} else {
			document.addEventListener('scroll', videoAutoPlay, false);
		}
	} else {
		document.removeEventListener('scroll', videoAutoPlay, false);
	}
}
;

/**
 * 引入处理逻辑文件
 */
/**
 * 1. ［统计］do_media_(un)like_callback 点击按钮后，客户端回调告知是否已订阅头条号
 * 2. page_state_change_callback的user_action 客户端广播是否已关注用户
 * 3. page_state_change_callback的pgc_action  客户端广播是否已订阅头条号
 * 4. ［实时］information-script-subscribe_switch 服务端告知是否已订阅头条号
 * 5. ［实时］客户端设置h5_extra.is_subscribed字段告知是否已订阅头条号
 * 6. ［实时］wenda_data设置is_concern_user字段告知是否已关注
 */
var change_following_state = (function() {
    var async_timer;
    var executing_target_state;

    function sync_func(willFollowing, callback) {
        var $header = $('header');
        var $button = $('.subscribe-button');

        executing_target_state = undefined;

        if (willFollowing) {
            $button.addClass('following').removeClass('disabled');
            $header.attr('fbs', 'following');
        } else {
            $button.removeClass('following disabled');
            $header.attr('fbs', '');
        }
    }

    return function(willFollowing, isAsync, callback) {
        // FIXME 那之前的回调和新的回调该怎么处理？这儿绝壁是个bug!!!
        if (typeof callback === 'function') {
            callback(willFollowing);
        }

        if (isAsync) {
            // NOTE 非实时修改状态，强制添加 450ms延时
            if (willFollowing !== executing_target_state) {
                clearTimeout(async_timer);
                executing_target_state = willFollowing;
                async_timer = setTimeout(sync_func, 450, willFollowing, callback);
            }
        } else {
            sync_func(willFollowing, callback);
        }
    }
})();


function bindStatisticsEvents2() {
    var $followButton = $('.subscribe-button'); // 由于PGC底部作者区域存在，会有多个关注按钮情形
    var $recommendList = $('#mediasug-list');

    // 关注点击，区分PGC与问答、帖子
    $followButton.on('click', followActionHandler);

    // 点击关注推荐的三角按钮
    $('.mediasug-arrow-button').on('click', mediasugArrowAction);

    // 点击关注推荐的卡片
    $recommendList.on('click', '.ms-item', mediasugCardClickHandler);

    // 点击关注推荐的关注按钮
    $recommendList.on('click', '.ms-subs', mediasugFollowAction);

    // 滚动过程中监测展示
    $recommendList.on('scroll', mediasugScroll.handler);
}



function followActionHandler(ev) {
    var $this = $(this);
    var userId = $this.data('userId');
    var mediaId = $this.data('mediaId');
    var isFollowing = $this.hasClass('following');
    var concernType = $this.attr('data-concerntype') || '';

    var pageType = globalArticleObject.article.type;
    var isTopButton = concernType === '';
    var isHitRecommend = globalArticleObject.hasExtraSpace && isTopButton;

    if ($this.hasClass('disabled')) {
        return;
    }
    $('.subscribe-button').addClass('disabled'); // TODO 此处改为属性实现？

    $('header').addClass('canmoving'); // TODO 此处改为属性实现？

    if (pageType === 'pgc') {
        doFollowMedia(userId, mediaId, isFollowing, concernType);

        if (isHitRecommend) {
            if (isFollowing) {
                $('header').attr('sugstate', 'no');
            } else {
                doRecommendUsers(h5_extra.media_user_id, recommendUsersSuccess, function() {});
            }
        }
    } else if (pageType === 'wenda') {
        doFollowUser(userId, mediaId, isFollowing);
    } else if (pageType === 'forum') {
        doFollowUser(userId, mediaId, isFollowing);
    }
}

function mediasugArrowAction(ev) {
    var $header = $('header');
    var willOpen = $header.attr('sugstate') === 'close';

    $header.attr('sugstate', willOpen ? 'open' : 'close');
    send_umeng_event('detail', willOpen ? 'click_arrow_down' : 'click_arrow_up', {
        extra: {
            source: 'article_detail'
        }
    });
    if (willOpen) {
        $('#mediasug-list').get(0).scrollLeft = 0;
    }
}

function mediasugCardClickHandler(ev) {
    if (!$(ev.target).is('.ms-subs')) {
        var userId = $(this).attr('it-is-user-id');

        send_umeng_event('detail', 'sub_rec_click', {
            value: userId,
            extra: {
                source: 'article_detail'
            }
        });

        window.location.href = 'sslocal://profile?uid=' + userId;
    }
}
function mediasugFollowAction(ev) {
    var $button = $(this);
    var isFollowing = $button.attr('isfollowing') != null;
    var userId = $button.closest('.ms-item').attr('it-is-user-id');
    var reason = $button.attr('reason');

    $button.attr('disabled', true);

    ToutiaoJSBridge.call('user_follow_action', {
        id: userId,
        action: isFollowing ? 'unfollow' : 'dofollow',
        reason: reason
    }, function(event) {
        $button.attr('disabled', null);
        if (typeof event === 'object' && event.code == 1) {
            send_umeng_event('detail', isFollowing ? 'sub_rec_unsubscribe' : 'sub_rec_subscribe', {
                value: userId,
                extra: {
                    source: 'article_detail'
                }
            });
            $button.attr('isfollowing', isFollowing ? null : '');
        }
    });
}

/***** 用户推荐部分 *****/
function domPrepare() {
    // 安卓上滑动时禁止带动屏幕滑动
    $('#mediasug-list').on('touchstart touchmove', function() {
        console.log('bytedance://disable_swipe');
    }).on('touchend touchcancel', function() {
        console.log('bytedance://enable_swipe');
    });

    // -webkit-overflow-scrolling: touch 会创建layer
    // 当其收起，覆盖在其上的部分仍然能够响应到滚动
    var _moc = document.querySelector('.mediasug-outer-container');
    var _mic = document.querySelector('.mediasug-inner-container');
    if (_moc && _mic) {
        _moc.addEventListener('transitionend', function(ev) {
            if (ev.target.offsetHeight == 0) { // 处于收起状态，隐藏dom元素
                _mic.style.display = 'none';
            }
        }, false);

        var MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
        if (MutationObserver) {
            var observer = new MutationObserver(function(mutations) {
                mutations.forEach(function(mutation) {
                    var _ma = mutation.attributeName;
                    if (_ma === 'sugstate') {
                        var _mav = mutation.target.getAttribute(_ma);
                        if (_mav === 'open') {
                            _mic.style.display = 'block';
                            console.info('SUG-OEPN');
                            mediasugScroll.handler();
                            $(document).on('scroll', mediasugScroll.pagescroll);
                        } else {
                            console.info('SUG-HIDE');
                            $(document).off('scroll', mediasugScroll.pagescroll);
                            mediasugScroll.pushimpr();
                        }
                    }
                });
            });

            observer.observe(document.getElementsByTagName('header')[0], {
                attributes: true
            });
        }
    }
}

function recommendUsersSuccess(list) {
    domPrepare();
    mediasugScroll.init(list);

    $('#mediasug-list-html').html(list.map(function(item) {
        return '<div class="ms-item" it-is-user-id="' + item.user_id + '"><div class="ms-avatar"><img class="ms-avatar-image" src="' + item.avatar_url + '"></div><div class="ms-name-wrap"><div class="ms-name' + (item.user_verified ? ' verified' : '') + '">' + item.name + '</div></div><div class="ms-desc">' + item.reason_description + '</div><button reason="' + item.reason + '" class="ms-subs"' + (item.is_following ? ' isfollowing ' : '') + (item.is_followed ? ' isfollowed ' : '') + '><span class="focus-icon">&nbsp;</span></button></div>';
    }).join('')).css('width', (list.length * 148 + 6 + 10) + 'px');

    $('header').attr('sugstate', 'open');
}
var doRecommendUsers = (function() {
    var _getRecommendUsersRequesting = false;
    var _cachedData = {};

    return function(to_user_id, success, errorf) {
        if (typeof success !== 'function' || typeof errorf !== 'function') {
            return;
        }

        if (_cachedData[to_user_id]) {
            success(_cachedData[to_user_id]);
            return;
        }

        $.ajax({
            dataType: 'jsonp',
            url: 'http://ic.snssdk.com/2/relation/follow_recommends',
            data: {
                to_user_id: to_user_id,
                page: 34
            },
            timeout: 1e4, // 10s
            beforeSend: function(xhr, settings) {
                if (_getRecommendUsersRequesting) {
                    return false; // abort
                }
                _getRecommendUsersRequesting = true;
            },
            success: function(json, status, xhr) {
                if (json.message === 'success'
                    && typeof json.data === 'object'
                    && Array.isArray(json.data.recommend_users)
                    && json.data.recommend_users.length >= 3) {
                    _cachedData[to_user_id] = json.data.recommend_users;
                    success(json.data.recommend_users);
                } else {
                    errorf(json, status, xhr);
                }
            },
            error: function(xhr, errorType, error) {
                errorf(error, errorType, xhr);
            },
            complete: function(xhr, status) {
                _getRecommendUsersRequesting = false;
            }
        });
    }
})();

var mediasugScroll = (function() {
    var containerWidth = innerWidth;
    var cardWidth = 148;

    var cardsList = {};
    var cardsCount = 0;

    var lastUmengIndex = 0;

    var inited = false;

    var lastIndex = [];

    var lastPos = 'in';

    return {
        init: function(list) {
            if (!inited) {
                inited = true;
                cardsList = list;
                cardsCount = list.length;
                this.imprcache = {};
                this.imprlog = [];
                // this.handler();
            }
        },
        range: function(left) {
            var start = Math.floor(left / cardWidth); // 从哪里展示
            start = Math.max(start, 0);
            left += containerWidth;
            var end = Math.ceil(left / cardWidth); // 显示到哪里
            end = Math.min(end, cardsCount) - 1;
            // console.info('range', start, end, lastIndex);

            var arr = [];
            while (start <= end) {
                arr[arr.length] = start++;
            }
            return arr;
        },
        pushimpr: function() {
            if (!inited) {
                return;
            }
            if (Object.keys(this.imprcache).length > 0) {
                for (var _uid in this.imprcache) {
                    var _s = this.imprcache[_uid];
                    this.imprlog.push({
                        uid: _uid,
                        time: _s,
                        duration: new Date().getTime() - _s
                    });
                    console.info('leave', _uid);
                }
                this.imprcache = {};
            }
            console.info('pushimpr', this.imprlog);
            if (this.imprlog.length > 0) {
                send_umeng_event('detail', 'sub_reco_impression_v2', {
                    value: globalArticleObject.author.userId,
                    extra: {
                        group_id: h5_extra.str_group_id,
                        impression: is_iphone() ? encodeURIComponent(JSON.stringify(mediasugScroll.imprlog)) : mediasugScroll.imprlog,
                        need_decode: is_iphone() ? 1 : 0
                    }
                });
                this.imprlog = [];
            }
        },
        dealimpr: function(drop, gogo) {
            var that = this;

            drop.forEach(function(idx) {
                var _uid = cardsList[idx]['user_id'];
                if (_uid in that.imprcache) {
                    var _s = that.imprcache[_uid];
                    that.imprlog.push({
                        uid: _uid,
                        time: _s,
                        duration: new Date().getTime() - _s
                    });
                    delete that.imprcache[_uid];
                    console.info('leave', _uid);
                }
            });
            gogo.forEach(function(idx) {
                var _uid = cardsList[idx]['user_id'];
                that.imprcache[_uid] = new Date().getTime();
                console.info('enter', _uid);
            });
        },
        handler: function(ev) {
            if (!inited) {
                return;
            }

            var thisIndex = mediasugScroll.range(this.scrollLeft || 0);
            var gogo = [];
            var lastMap = {};

            for (var i = 0; i < lastIndex.length; i++) {
                lastMap[lastIndex[i]] = true;
            }
            for (i = 0; i < thisIndex.length; i++) {
                if (thisIndex[i] in lastMap) {
                    delete lastMap[thisIndex[i]];
                } else {
                    gogo.push(thisIndex[i]);
                }
            }

            var drop = Object.keys(lastMap);

            mediasugScroll.dealimpr(drop, gogo);
            lastIndex = thisIndex;
        },
        pagescroll: function(ev) {
            if (!inited) {
                return;
            }
            var thisdom = $('#mediasug-list').get(0);

            var pos = 'in';
            var coords = thisdom.getBoundingClientRect();
            if (coords.bottom <= 0 || coords.top > (window.innerHeight || document.body.clientHeight)) {
                pos = 'out';
            }

            if (pos === 'in' && lastPos === 'out') {
                console.info('IN');
                // 展示出来了，利用上次的结果直接做新的即可。
                mediasugScroll.dealimpr([], lastIndex);
            } else if (pos === 'out' && lastPos === 'in') {
                console.info('OUT');
                // 滑出屏幕，结算一次。
                mediasugScroll.pushimpr();
            }
            lastPos = pos;
        }
    };
})();


/***** 关注部分 *****/
/**
 * 关注事件统计、头像点击统计
 * @return {[type]} [description]
 */
var subscribeTimeoutTimer;

function doFollowUser(userId, mediaId, isFollowing, reason) {
    // NOTE iOS5.7.2版本前，无网状态下点击按钮，客户端不会回调。所以需要web做一个点击
    //      超时的兼容。
    subscribeTimeoutTimer = setTimeout(change_following_state, 1e4, isFollowing, true);

    ToutiaoJSBridge.call('user_follow_action', {
        id: userId,
        action: isFollowing ? 'unfollow' : 'dofollow',
        reason: reason
    }, function(event) {
        clearTimeout(subscribeTimeoutTimer);
        if (!event) {
            change_following_state(isFollowing, true);
        } else if (typeof event === 'object' && event.code == 1) {
            change_following_state(!!event.status, true);
        } else {
            change_following_state(isFollowing, true);
        }
    });
}

function doFollowMedia(userId, mediaId, isFollowing, concernType) {
    var isTopButton = concernType === '';
    var isHitRecommend = globalArticleObject.hasExtraSpace && isTopButton;

    // NOTE iOS5.7.2版本前，无网状态下点击按钮，客户端不会回调。所以需要web做一个点击
    //      超时的兼容。
    subscribeTimeoutTimer = setTimeout(change_following_state, 1e4, isFollowing, true);

    ToutiaoJSBridge.call(isFollowing ? 'do_media_unlike' : 'do_media_like', {
        id: mediaId,
        uid: userId,
        concern_type: concernType
    }, function(event) {
        clearTimeout(subscribeTimeoutTimer);
        // NOTE iOS 5.7.2版本以前，由于对回调处理有bug导致event.code恒为0。按
        //      钮状态生效实际是因为page_state_change方法的监听。这样导致了当
        //      网络状况差时，由于很快就返回了0去掉了disabled，转圈出现很短时间就
        //      消失了（也就是闪了一下）。而过段时间请求回来，通过psc同步状态，
        //      此时才会切换状态（也就是感觉慢）。故针对旧版本，忽略对code0的处理。
        if (event.code == 1) {
            change_following_state(!isFollowing, true, function(willFollowing) {
                if (willFollowing) {
                    send_umeng_event('preview', 'preview_click_sub');
                    // 老一些的版本没有showToast这个字段
                    // FIXME 怎么看这个toast都应该是客户端自己弄
                    if (!('showToast' in event) || event.showToast) {
                        if (!isHitRecommend) {
                            ToutiaoJSBridge.call('toast', {
                                text: '将增加推荐此头条号内容',
                                icon_type: 'icon_success'
                            });
                        }
                    }
                } else {
                    send_umeng_event('preview', 'preview_click_cancel_sub');
                }
            });
        } else if (is_android || is_app_version_bigger_than('5.7.2')) {
            change_following_state(isFollowing, true);
        } else {
            change_following_state(isFollowing, true);
        }
    });
}
;
/**
 * 处理转发文案
 * 1. 人名加高亮
 * 2. hashtag
 */
function encodeHTML (string) {
    return string.replace(/"/g, '&quot;').replace(/'/g, '&#39;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
}
function decodeHTML (string) {
    return string.replace(/&quot;/g, '\"').replace(/&#39;/g, '\'').replace(/&lt;/g, '<').replace(/&gt;/g, '>');
}
function getBrPosition (html) {
    var notagHTML = decodeHTML(html);
    var i = 0;
    var notagArr = notagHTML.split('<br>');
    return notagArr.map(function (str) {
        var o = str.length + i;
        i += str.length;
        return {
            isBR: true,
            pos: o
        };
    }).slice(0, notagArr.length - 1);
}
function processThreadRepostContent () {
    $('p[data-rich-span]').each(function(_, dom) {
        var links;
        var offset = 0;
        var pContent = dom.textContent;
        var pHTML = dom.innerHTML;
        var pRichContent = '';

        try {
            links = JSON.parse(dom.dataset.richSpan).links;
            // 还要按start排序？
            var _starts = [];
            var _indexs = {};
            links.forEach(function (link, index) {
                _starts.push(link.start);
                _indexs[link.start] = link;
            });


            var _brs = getBrPosition(pHTML);
            _brs.forEach(function (br, index) {
                if (_indexs[br.pos]) {
                    // br和富文本位置重合
                    _indexs[br.pos] = [br, _indexs[br.pos]];
                } else {
                    _starts.push(br.pos);
                    _indexs[br.pos] = br;
                }
            });

            links = [];

            _starts.sort(function (a, b) {
                return a - b;
            }).forEach(function (start) {
                links.push(_indexs[start]);
            });
        } catch (ex) {
            // do nothing.
        }

        if (!Array.isArray(links) || links.length === 0) {
            return;
        }

        links.forEach(function (link) {
            if (Array.isArray(link)) {
                // 既有br，又有a
                pRichContent += encodeHTML(pContent.substring(offset, link[0].pos));
                pRichContent += '<br>';
                pRichContent += '<a href="' + link[1].link + '">';
                pRichContent += encodeHTML(pContent.substr(link[1].start, link[1].length));
                pRichContent += '</a>';
                offset = link[1].start + link[1].length;
            } else if (!link.isBR) {
                pRichContent += encodeHTML(pContent.substring(offset, link.start));
                pRichContent += '<a href="' + link.link + '">';
                pRichContent += encodeHTML(pContent.substr(link.start, link.length));
                pRichContent += '</a>';
                offset = link.start + link.length;
            } else {
                pRichContent += encodeHTML(pContent.substring(offset, link.pos));
                pRichContent += '<br>';
                offset = link.pos;
            }
        });
        pRichContent += encodeHTML(pContent.substr(offset));

        fastdom.mutate(function () {
            dom.innerHTML = pRichContent;
        });
    });
}

/**
 * 转发帖子原文
 * 如原贴被删展示错误信息
 * 加作者信息
 */
function processRepostThread () {
    $('.tt-repost-thread').each(function(_, dom) {
        var ds = dom.dataset;
        var originUser = '<a class="originuser" href="' + ds.userSchema + '">@' + ds.userName + '</a>：';
        if (globalArticleObject.forum_extra.origin_forum_info) {
            originUser += '<a href="' + globalArticleObject.forum_extra.origin_forum_info.schema + '">#' + globalArticleObject.forum_extra.origin_forum_info.name + '#</a>';
        }
        if (ds.title) {
            originUser += '【' + ds.title + '】';
        }
        var originUserId = 0;
        if (/\d+/.test(ds.userSchema)) {
            originUserId = ds.userSchema.match(/\d+/)[0];
        }

        ToutiaoJSBridge.call('appInfo', {}, function (json) {
            var json_user_id = typeof json === 'object' ? json.user_id : -1;
            // 0 删除
            // 1 帖子是所有人可见状态
            // 2 帖子是仅自己可见状态
            // 4 帖子正在审核中
            fastdom.mutate(function () {
                if (ds.showOrigin == 0) {
                    $(dom).html(ds.showTips || '原内容已经删除').removeClass('tt-repost-thread').addClass('tt-repost-delete');
                } else {
                    var maybeP = $(dom).find('p[data-rich-span]');
                    if (maybeP.length === 0) {
                        $(dom).prepend('<p class="first-forum-p">' + originUser + '</p>');
                    } else {
                        maybeP.prepend(originUser).addClass('first-forum-p');
                    }

                    $(dom).on('click', function (ev) {
                        if ($(ev.target).closest('.image').length === 0 && $(ev.target).closest('.originuser').length === 0) {
                            location.href = ds.openUrl;
                        }
                    });
                }
                dom.style.visibility = 'visible';
            });
        });
    });
}

/**
 * 转发文章
 */
function processRepostArticle () {
    var ArticleLinkTemplateFunction = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='';
 if (data.showOrigin == 0) { 
__p+='<div class="tt-repost-delete">'+
((__t=( data.showTips ))==null?'':__t)+
'</div>';
 } else { 
__p+='<a class="tt-repost-article" href="'+
((__t=( data.openUrl ))==null?'':__t)+
'"><div class="coverwrap"><div class="cover" image="'+
((__t=( data.cover ))==null?'':__t)+
'" type="'+
((__t=( data.articleType ))==null?'':__t)+
'" style="background-image: url('+
((__t=( data.cover ))==null?'':__t)+
');"></div></div><div class="titlewrap"><div class="title">'+
((__t=( data.author ))==null?'':__t)+
'：'+
((__t=( data.title ))==null?'':__t)+
'</div></div></a>';
 } 
__p+='';
}
return __p;
};
    $('tt-repost-article').each(function (__, dom) {
        // dom.dataset.status=31;
        // dom.dataset.reason='删啦删啦';
        // dom.dataset.type = 'video';
        // dom.dataset.cover = '';
        var $newdom = $(ArticleLinkTemplateFunction({ data: dom.dataset }));

        fastdom.mutate(function () {
            $(dom).replaceWith($newdom);
        });
    });
}

/**
 * 电影评分
 */
function processFilm () {
    if (window.forum_extra && 'publish_score' in window.forum_extra) {
        var score = window.forum_extra.publish_score;
        var star = Math.ceil(score);
        var html = '<div class="film-star-score-wrapper"><span class="iconfont film-star-score" data-score="' + star + '">' + score + '</span></div>';
        fastdom.mutate(function () {
            $('p').eq(0).append(html);
        });
    }
}
;
/**
 * 将页面上的音频模板转换为可以播放的audio标签和控制器
 * @return {undefined}
 */
function getAudioSourceById (id, callback, error) {
    $.ajax({
        type: 'GET',
        dataType: 'jsonp',
        url: 'http://i.snssdk.com/audio/urls/1/toutiao/mp4/' + id,
        error: error,
        success: function (json, status, xhr) {
            try {
                callback(atob(json.data.audio_list.audio_1.main_url.replace(/\n/gi, ''))); // Safari上对\n处理有bug
            } catch (err) {
                error();
            }
        }
    });
}
function sendAudioEvent (tag, label) {
    send_umeng_event(tag, label, {
        value: h5_extra.str_item_id,
        extra: {
            sound_id: this.audioId,
            percent: this.currentTime / this.duration,
            duration: this.duration,
            current_time: this.currentTime
        }
    });
}
function processAudio () {
    var MusicTemplateFunction = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='<div class="musicplayer" play-state="not-playing"><!-- <div class="iconfont music-status"></div> --><div class="music-state"><div class="music-info"><span class="music-name">'+
((__t=(music.name))==null?'':_.escape(__t))+
'</span><span class="music-time">'+
((__t=(music.time))==null?'':_.escape(__t))+
'</span></div><div class="music-musician">'+
((__t=(music.musician))==null?'':_.escape(__t))+
'</div></div><div class="progressbar"></div><audio preload="none" duration="'+
((__t=(music.duration))==null?'':_.escape(__t))+
'" audioId="'+
((__t=(music.audioId))==null?'':_.escape(__t))+
'"></audio></div>';
}
return __p;
};
    $('tt-audio').each(function (index, dom) {
        var music = {
            audioId: dom.getAttribute('data-id'),
            name: dom.getAttribute('title'),
            duration: +dom.getAttribute('time'),
            time: formatDuration(+dom.getAttribute('time')),
            musician: dom.getAttribute('content')
        };

        var $newdom = $(MusicTemplateFunction({music: music}));
        $(dom).replaceWith($newdom);

        // 渲染完毕，立即获取src
        getAudioSourceById(music.audioId, function(src){
            $newdom.find('audio').attr('src', src);
        }, function () {
            // do nothing.
        });

        // play单提出来
        function _play (player, otherPlayer) {
            if (otherPlayer) {
                sendAudioEvent.call(otherPlayer, 'sound_over_others', 'stop_play');
                otherPlayer.pause();
            }
            player.play();
            send_umeng_event('sound', 'detail_play', {
                value: h5_extra.str_item_id,
                extra: {
                    sound_id: player.audioId
                }
            });
        }

        $newdom.on('click', function (ev) {
            var $this = $(this);
            var player = $this.find('audio').get(0);
            var otherPlayer = $('[play-state="playing"]').find('audio').get(0);
            player.audioId = music.audioId;

            if (!player.src) {
                getAudioSourceById(player.audioId, function(src){
                    player.setAttribute('src', src);
                    // FIXME 这个延时不太好，但不延时又不能播，
                    // 并且直接导致play和playingState是不能做到同步的，怎么搞？！
                    setTimeout(function(){
                        _play(player, otherPlayer);
                    }, 500);
                }, function () {
                    ToutiaoJSBridge.call('toast', {
                        text: '音频获取失败，请重试',
                        icon_type: 'icon_error'
                    });
                });
            } else {
                if (player.paused) {
                    _play(player, otherPlayer);
                } else {
                    sendAudioEvent.call(player, 'sound', 'stop_play');
                    player.pause();
                }
            }
        }).find('audio').on('timeupdate', function(ev){
            if (this.currentTime >= this.duration) {
                this.pause();
            } else {
                $newdom.find('.progressbar').css('width', this.currentTime/this.duration*100 + '%');
            }
        }).on('durationchange', function(ev){
            $(this).closest('.musicplayer').find('.music-time').text(formatDuration(this.duration));
        }).on('playing', function(ev){
            $(this).closest('.musicplayer').attr('play-state', 'playing');
        }).on('pause', function(ev){
            if (this.duration - this.currentTime < 0.3) {
                // 可能存在跳帧，所以小于0.3秒（经验值）就可以认为是播放结束
                sendAudioEvent.call(this, 'sound_over', 'end_play');
            }
            this.currentTime = 0; // 每次‘暂停’都相当于停止
            $(this).closest('.musicplayer').attr('play-state', 'not-playing');
        });
    });
}


/**
 * 处理问答页内点赞逻辑
 * @return {undefined}
 * @TODO 问题在于如果5.6点踩了，回到5.5上是不展示踩按钮的，此时会展示成功
 */
function processDigg () {
    $('#digg').on('click', function(){
        if ($(this).attr('wenda-state') === 'digged') {
            ToutiaoJSBridge.call('toast', {
                text: '你已经赞过',
                icon_type: 'icon_error'
            });
        } else if ($('#bury').attr('wenda-state') === 'buryed') {
            ToutiaoJSBridge.call('toast', {
                text: '你已经踩过',
                icon_type: 'icon_error'
            });
        } else {
            ToutiaoJSBridge.call('page_state_change', {
                type: 'wenda_digg',
                id: $(this).attr('data-answerid'),
                status: 1 // 1表示将要点赞
            });
        }
    });
    $('#bury').on('click', function(){
        if ($(this).attr('wenda-state') === 'buryed') {
            ToutiaoJSBridge.call('toast', {
                text: '你已经踩过',
                icon_type: 'icon_error'
            });
        } else if ($('#digg').attr('wenda-state') === 'digged') {
            ToutiaoJSBridge.call('toast', {
                text: '你已经赞过',
                icon_type: 'icon_error'
            });
        } else {
            ToutiaoJSBridge.call('page_state_change', {
                type: 'wenda_bury',
                id: $(this).attr('data-answerid'),
                status: 1 // 1表示将要踩
            });
        }
    });
}

/**
 * 商业化按钮组
 */
function processPhoneGroup() {
    $('tt-phone-group').each(function(index, dom) {
        var $dom = $(dom);
        var $newdom = $('<div class="cpg-container">');
        var buttonCount = 0;
        var hasCall = false;
        var hasUrl = false;

        if ($dom.attr('contact-phone') && $dom.attr('contact-name')) {
            hasCall = true;
            buttonCount++;
            $newdom.append('<a class="cpg-button cpg-call" href="tel:' + $dom.attr('contact-phone') + '">' + $dom.attr('contact-name') + '</a>');
            $newdom.attr('button-count', buttonCount);
        }
        if ($dom.attr('book-url') && $dom.attr('book-name')) {
            hasUrl = true;
            buttonCount++;
            $newdom.append('<a class="cpg-button cpg-link" href="sslocal://webview?url=' + encodeURIComponent($dom.attr('book-url')) + '">' + $dom.attr('book-name') + '</a>');
            $newdom.attr('button-count', buttonCount);
        }
        $newdom.on('click', '.cpg-button', function(ev) {
            var isCall = $(this).hasClass('cpg-call');
            send_umeng_event('embeded_button_ad', 'click', {
                value: h5_extra.str_item_id,
                extra: {
                    action_type: isCall ? 'call' : 'url',
                    button_value: isCall ? $dom.attr('contact-phone') : $dom.attr('book-url'),
                    action_time: (new Date()).getTime()
                }
            });
        });

        $dom.replaceWith($newdom);

        setTimeout(function(){
            send_exposure_event_once($newdom.get(0), function() {
                if (hasUrl) {
                    send_umeng_event('embeded_button_ad', 'show', {
                        value: h5_extra.str_item_id,
                        extra: {
                            action_type: 'url',
                            button_value: $dom.attr('book-url'),
                            action_time: (new Date()).getTime()
                        }
                    });
                }
                if (hasCall) {
                    send_umeng_event('embeded_button_ad', 'show', {
                        value: h5_extra.str_item_id,
                        extra: {
                            action_type: 'call',
                            button_value: $dom.attr('contact-phone'),
                            action_time: (new Date()).getTime()
                        }
                    });
                }
            }, true);
        }, 200);
    });
}

/**
 * 调整正文中的可搜索关键字链接
 * @return {undefined}
 */
function processProHref () {
    $('[pro-href]').each(function() {
        var $this = $(this);
        var starhref = $this.attr('starhref');

        if (starhref) {
            if (globalArticleObject.useServerV) {
                try {
                    var auth_type = JSON.parse($this.attr('user-auth-info')).auth_type;
                    $this.addClass('new-v').append(buildServerVIcon(auth_type, 'label_icon'));
                } catch (err) {}
            }
            $this.attr('href', starhref);

            var maybeUid = starhref.match(/\d+/i);
            if (maybeUid) {
                maybeUid = +maybeUid[0];
            } else {
                maybeUid = 0;
            }

            $this.on('click', function() {
                send_umeng_event('star_words', 'click', {
                    value: maybeUid
                });
            });
            send_exposure_event_once(this, function() {
                send_umeng_event('star_words', 'show', {
                    value: maybeUid
                });
            }, true);
        } else {
            $this.attr('href', $this.attr('pro-href'));

            $this.on('click', function() {
                send_umeng_event('highlight_words', 'click', {
                    value: window.h5_extra.str_group_id,
                    extra: {
                        highword: encodeURIComponent($this.text())
                    }
                });
            });
            send_exposure_event_once(this, function() {
                send_umeng_event('highlight_words', 'show', {
                    value: window.h5_extra.str_group_id,
                    extra: {
                        highword: encodeURIComponent($this.text())
                    }
                });
            }, true);
        }
    });
}


/**
 * 优化正文中的表格交互行为，iOS上开启横滑模式
 * @param {boolean} horizontal_open 是否开启横向滚动
 * @return {undefined}
 * NOTE 安卓不能局部bytedance://disable_swipe，故右滑手势会让整个页面退出
 *      安卓部分系统不触发touchend，不方便实现touchend时swipe.style.opacity = 1
 */
function processTable(horizontal_open) {
    $('table').each(function (index, dom) {
        var $this = $(this);
        // if ($this.find('.image').length === 0) {
            $this.addClass('border');
            if (horizontal_open) {
                if (is_iphone()) {
                    $this.wrap('<div class="table-wrap horizontal_scroll"/>');
                    var p = $this.parent();
                    if ($this.width() > maxWidth) {
                        p.append('<div class="swipe_tip">左滑查看更多</div>');
                        p.bind('touchstart', function(){
                            this.querySelector('.swipe_tip').style.opacity = '0';
                        }).bind('scroll touchend', function(){
                            if (this.scrollLeft == 0) {
                                this.querySelector('.swipe_tip').style.opacity = '1';
                            }
                        });
                    }
                } else if (is_android()) {
                    $this.wrap('<div class="table-wrap horizontal_scroll_android"/>');
                }
            }
        // }
    });
}


/**
 * 视频封面图加载成功，调整父节点背景为纯黑
 * @return {undefined}
 */
function appendVideoImg() {
    var parent = this.parentNode;
    if (parent) {
        parent.style.background = '#000';
    }
}


/**
 * 视频封面图加载失败，删除<img>标签
 * @return {undefined}
 */
function errorVideoImg() {
    var parent = this.parentNode;
    if (parent) {
        parent.removeChild(this);
    }
}


/**
 * 显示页面中的视频
 * @return {undefined}
 */
function processCustomVideo() {
    $('.custom-video').each(function (idx, cv) {
        var $cv = $(this);
        var dataset = this.dataset;

        var cvw = dataset.width;
        var cvh = dataset.height;

        var max_ratio = 75; //最大展示height:width比，超过时高度压缩，水平方向左右两侧留黑色背景
        var rel_ratio = 0;
        var ratio = max_ratio;
        var style = '';

        if (cvw && cvh) {
            rel_ratio = (100 * cvh / cvw).toFixed(2);
            if (rel_ratio <= max_ratio) {
                ratio = rel_ratio;
            } else {
                style = 'height: 100%; width: auto;';
            }
        }

        $cv.css('padding-bottom', ratio + '%');

        if (dataset.wendaSource && typeof window.wenda_extra === 'object') {
            var duration = formatDuration(dataset.duration);
            $cv.html('<img src="' + dataset.poster + '" style="' + style + '" onload="appendVideoImg.call(this)" onerror="errorVideoImg.call(this)" /><i class="custom-video-trigger"></i><i class="custom-video-duration">' + duration + '</i>');

            if(dataset.wendaSource === 'pgc'){
                var $userInfo = $('<a class="cv-wd-info-wrapper" href="' + dataset.openUrl + '"><span class="cv-wd-info-name" ' + (Boolean(Number(dataset.isVerify)) ? 'is-verify' : '') + '>' + dataset.mediaName + '</span><span class="cv-wd-info-pc">' + dataset.playCount + '次播放</span></a>');
                $userInfo.on('click', function() {
                    ToutiaoJSBridge.call('pauseVideo');
                    send_umeng_event('answer_detail', 'click_video_detail', {
                        value: wenda_extra.ansid,
                        extra: {
                            video_id: dataset.vid,
                            enter_from: wenda_extra.enter_from || '',
                            ansid: wenda_extra.ansid,
                            qid: wenda_extra.qid,
                            parent_enterfrom: wenda_extra.parent_enterfrom || ''
                        }
                    });
                })
                $cv.after($userInfo);
            }

            send_exposure_event_once(this, function() {
                send_umeng_event('video_show', 'click_question_and_answer', {
                    value: wenda_extra.ansid,
                    extra: {
                        position: 'detail',
                        video_id: dataset.vid,
                        enter_from: wenda_extra.enter_from || '',
                        ansid: wenda_extra.ansid,
                        qid: wenda_extra.qid,
                        parent_enterfrom: wenda_extra.parent_enterfrom || ''
                    }
                });
            }, true);
        } else {
            $cv.html('<img src="' + dataset.poster + '" style="' + style + '" onload="appendVideoImg.call(this)" onerror="errorVideoImg.call(this)" /><i class="custom-video-trigger"></i>');
        }
    }).on('click', function() {
        playVideo(this, 0);
    });
}


/**
 * 告知客户端顶部区域是否显示
 * @param  {object} event 滚动事件
 * @return {undefined}
 */
function checkDisplayedFactory(selector, callBirdgeName) {
    lastBottom = {};

    return function() {
        var dom = document.querySelector(selector);

        if (!dom) {
            return;
        }

        var coords = dom.getBoundingClientRect();

        if (coords.bottom < 0 && lastBottom[selector] >= 0) {
            ToutiaoJSBridge.call(callBirdgeName, {
                show: true
            });
        } else if (coords.bottom >= 0 && lastBottom[selector] < 0) {
            ToutiaoJSBridge.call(callBirdgeName, {
                show: false
            });
        }

        lastBottom[selector] = coords.bottom;
    }
}
var checkHeaderDisplayed = checkDisplayedFactory('#profile', 'showTitleBarPgcLayout');
var checkWendanextDisplayed = checkDisplayedFactory('.serial', 'showWendaNextLayout');
var checkWendaABHeaderLayout = checkDisplayedFactory('.wenda-title', 'showWendaABHeaderLayout');

/**
 * 处理转载推荐，主要逻辑分为两部
 * 首先要“尽快”在页面上插入转载容器，防止页面显示后再插入抖动；
 * 其次根据内容计算显示长度，要保证“推荐此文”露出，不能截断人名。这个过程可以"适当"慢一些。
 */
function processZZComments() {
    if (!Array.isArray(window.zz_comments)) {
        return;
    }
    var zzcomments = window.zz_comments;

    window.send_zz_umeng = function (media_id) {
        send_umeng_event('detail', 'zz_comment_click', {
            value: window.h5_extra.str_group_id,
            ext_value: media_id
        });
    };

    var shownMedias = [];
    var $rot = $('.zzcomments');
    var $dom = $('.rec-us');
    var $end = $('.rec-end');

    var domMaxWidth = $rot.width() - 15 - $end.width() - 15;

    var pIndex = 0;
    var _tobj = zzcomments[pIndex].media_info;

    var $tmp = $('<a class="rec-u" onclick="send_zz_umeng(\'' + _tobj.media_id + '\');" href="bytedance://media_account?from=zzcomments&media_id=' + _tobj.media_id + '">' + _tobj.name + '</a>');
    $dom.append($tmp);
    shownMedias.push(_tobj.media_id.toString());

    if ($dom.width() > domMaxWidth) {
        $rot.prepend($end).addClass('oneauthor'); // 第一个转载者的名字长度已然过长
    } else {
        // NOTE 由于第一个转载信息和其后的做法不一致，所以不能用do/while简化逻辑
        for (pIndex = 1; pIndex < zzcomments.length; pIndex++) {
            _tobj = zzcomments[pIndex].media_info;
            if (shownMedias.indexOf(_tobj.media_id.toString()) > -1) {
                continue;
            }
            var $dot = $('<span>、</span>');
            var $tmp = $('<a class="rec-u" onclick="send_zz_umeng(\'' + _tobj.media_id + '\');" href="bytedance://media_account?from=zzcomments&media_id=' + _tobj.media_id + '">' + _tobj.name + '</a>');
            $dom.append($dot);
            $dom.append($tmp);

            if ($dom.width() > domMaxWidth) {
                $dot.remove();
                $tmp.remove();
                break;
            } else {
                shownMedias.push(_tobj.media_id.toString());
            }
        }
    }

    $end.css('visibility', 'visible');
    send_umeng_event('detail', 'show_zz_comment', {
        value: window.h5_extra.str_group_id,
        ext_value: window.h5_extra.str_item_id,
        extra: {
            media_ids: shownMedias.join(',')
        }
    });
}


/**
 * 处理小说目录的点击
 */
function processNovel () {
    if (!('novel_data' in window.h5_extra)) {
        return;
    }

    var _statistics = {
        value: window.h5_extra.str_group_id,
        extra: {
            item_id: window.h5_extra.str_item_id
        }
    }

    $('body').on('click', '#prev_serial_link', function(ev){
        send_umeng_event('detail', 'click_pre_group', _statistics);
    }).on('click', '#next_serial_link', function(ev){
        send_umeng_event('detail', 'click_next_group', _statistics);
    }).on('click', '#index_serial_link', function(ev){
        send_umeng_event('detail', 'click_catalog', _statistics);
    });
}


/**
 * 如果中了AB测，不管有没有菜单数据都可以展示copy-authorbar
 * 如果内容不足一屏幕，不应当显示出来
 */
function processPgcBottomCard(){
    // 接口迁移，逻辑移动到context
}


/**
 * 监听客户端广播的页面变更状态，更新页面相关元素状态
 * @param  {object} event 事件对象
 * @return {undefined}
 */
function processPageStateChangeEvent (event) {
    switch (event.type) {
        case 'pgc_action':  // 通过mediaId派发，我应通过mediaId寻找
            console.info('pgc_action', event); // {type: 'pgc_action', id: '1234567', status: 1}
            if (subscribeTimeoutTimer) {
                clearTimeout(subscribeTimeoutTimer);
            }
            var $followButton = $('.subscribe-button');
            var currentPageAuthorMediaId = $followButton.data('mediaId');
            if (event.id == currentPageAuthorMediaId && 'status' in event) {
                change_following_state(!!event.status, true);
            }
            break;

        case 'user_action': // 问答、帖子关注作者
            console.info('user_action', event);
            // NOTE android在5.7.10之前不会执行user_follow_action的回调方法
            if (subscribeTimeoutTimer) {
                clearTimeout(subscribeTimeoutTimer);
            }
            var $followButton = $('.subscribe-button');
            var currentPageAuthorUserId = $followButton.data('userId');
            if (event.id == currentPageAuthorUserId && 'status' in event) {
                change_following_state(!!event.status, true);
            } else { // 推荐关注也存在状态同步
                var $maybeuser = $('[it-is-user-id="' + event.id + '"]');
                if ($maybeuser.length > 0 && 'status' in event) {
                    $maybeuser.find('.ms-subs').attr('isfollowing', !!event.status ? '' : null);
                }
            }
            break;
        case 'wenda_digg': // 问答点赞
            var currentPageAnswerId = $('#digg').attr('data-answerid');
            // NOTE iOS 此处会有运行时bug：点赞可能会再极端情况成功但回传error
            //      故用当前页面状态处理 (event.status == 1)
            if (event.id == currentPageAnswerId && $('#digg').attr('wenda-state') !== 'digged') {
                $('#digg').attr('wenda-state', 'digged');
                var currentDiggCount = +$('#digg').find('.digg-count').attr('realnum');
                formatCount('.digg-count', currentDiggCount+1, '赞');
                formatCount('.digg-count-special', currentDiggCount+1, '0');
            }
            break;
        case 'wenda_bury': // 问答踩
            var currentPageAnswerId = $('#bury').attr('data-answerid');
            if (event.id == currentPageAnswerId && $('#bury').attr('wenda-state') !== 'buryed') {
                $('#bury').attr('wenda-state', 'buryed');
                var currentBuryCount = +$('#bury').find('.bury-count').attr('realnum');
                formatCount('.bury-count',currentBuryCount+1, '踩');
            }
            break;
        case 'forum_action':
            var $maybebutton = $('.card.fiction').find('.button');
            var currentPageForumId = $maybebutton.attr('forum-id');
            if (event.id == currentPageForumId) {
                $maybebutton.attr('is-concerned', Boolean(event.status))
                    .html(event.status ? '已关注' : '关注');
            }
            break;
        case 'concern_action':
            var $maybebutton = $('.card.fiction').find('.button');
            var currentPageConcernId = $maybebutton.attr('concern-id');
            if (event.id == currentPageConcernId) {
                $maybebutton.attr('is-concerned', Boolean(event.status))
                    .html(event.status ? '已关注' : '关注');
            }
            break;
        case 'carousel_image_switch':
            // 客户端轮播图切换事件
            if (typeof onCarouselImageSwitch === 'function') {
                if ('forum_extra' in window && window.forum_extra.thread_id == event.id) {
                    onCarouselImageSwitch(event.status);
                } else if ('wenda_extra' in window && window.wenda_extra.ansid == event.id) {
                    onCarouselImageSwitch(event.status);
                } else if (h5_extra.str_group_id == event.id) {
                    onCarouselImageSwitch(event.status);
                }
            }
            break;
        case 'block_action':
            // 拉黑事件，已关注的人要取消关注
            // 先看上下两个按钮的，再看关注推荐的
            console.info(event);
            if (event.status == 1) {
                var $followButton = $('.subscribe-button');
                var currentPageAuthorUserId = $followButton.data('userId');

                if (event.id == currentPageAuthorUserId) {
                    change_following_state(false, true);
                } else {
                    var $maybeuser = $('[it-is-user-id="' + event.id + '"]');
                    if ($maybeuser.length > 0) {
                        $maybeuser.find('.ms-subs').attr('isfollowing', null);
                    }
                }
            }
            break;
    }
}


function bindStatisticsEvents () {
    // 点击头像、作者区域
    $(document.body).on('click', '.pgc-link', function () {
        if (globalArticleObject.article.type === 'forum') {
            send_umeng_event('talk_detail', 'click_ugc_header', globalArticleObject.forumStatisticsParams);
        } else if (globalArticleObject.article.type === 'pgc') {
            send_umeng_event('detail', 'click_pgc_header', {
                value: h5_extra.media.id,
                extra: {
                    item_id: h5_extra.str_item_id
                }
            });
        }
    });

    bindStatisticsEvents2();
}


/**
 * 渲染时间统计
 */
window.addEventListener('load', function() {
    if (!window.performance) return;
    var t = window.performance.timing;
    var times = {
        rec: t.domLoading - t.fetchStart, // 检测从一开始到开始解析所需的时间，讲道理应该很短
        dcle: t.domContentLoadedEventEnd - t.domLoading, // 标记 DOM 准备就绪并且没有样式表阻碍 JavaScript 执行的时间点 - 意味着我们可以开始构建呈现树了
        dc: t.domComplete - t.domLoading, // 所有的处理完成，网页上所有资源（图片等）下载完成
    };

    // $('body').append('<div style="position: fixed;z-index: 1;left: 0;top: 0;">' + JSON.stringify(times) + '</div>');

    send_umeng_event('article_fe', 'performance_timing', {
        varlue: h5_extra.str_group_id,
        extra: times
    });
});

/**
 * 处理段落排版
 * NOTE 短期做法如下，后续调研更精细的处理方式
 * 纯英文段落按『左对齐，单词截断』处理
 * 存在中文字符，按『两端对齐，加连字符』处理
 * 若中英混排时存在超过30+的连续英文，则在其前断行
 */
function processParagraph() {
    var chineseRE = /[\u2e80-\u2eff\u3000-\u303f\u3200-\u9fff\uf900-\ufaff\ufe30-\ufe4f]/;
    var longEngRE = /[a-z0-9_:\-\/.%]{26,}/ig;
    var isHUAWEI = /huawei/.test(navigator.userAgent.toLowerCase());

    // if (is_iphone()) {
    //     document.body.classList.add('iphone');
    // } else if (is_android()) {
    //     document.body.classList.add('android');
    // }

    if (isHUAWEI) {
        document.body.classList.add('huawei');
    }

    $('article p').each(function(x, p) {
        // NOTE 如段落里有图片，则不处理，因为处理需要操作dom，图片同样操作dom
        // 现有的写法对二者不可控，会发生严重问题
        if (p.classList.contains('pgc-img-caption') || !p.textContent || $(p).find('.image').length > 0) {
            return;
        }

        if (!chineseRE.test(p.textContent)) {
            p.style.textAlign = 'left';
            // p.style.wordBreak = 'break-word';
        } else if (longEngRE.test(p.textContent)) { // 会有去strong的问题？？？
            var retar = p.textContent.match(longEngRE);
            retar.forEach(function(str){
                p.innerHTML = p.innerHTML.replace(str, function (match) {
                    return '<br class="sysbr">' + match;
                });
            });
        }
    });
}

function processGameDownloader () {
    var render = function(obj){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
with(obj||{}){
__p+='';
 if (data.card_type === '1') { 
__p+='<a class="game-downloader gd1" href="'+
((__t=( data.detail ))==null?'':__t)+
'"><img class="gd1-icon" src="'+
((__t=( data.logo ))==null?'':__t)+
'"><div class="gd-button gd1-btn"><i class="iconfont">'+
((__t=( data.wordsMap.font[data.installed] ))==null?'':__t)+
'</i>'+
((__t=( data.wordsMap.thin[data.installed] ))==null?'':__t)+
'</div><div class="gd1-cont"><div class="gd1-cont-name">'+
((__t=( data.name ))==null?'':__t)+
'</div><div class="gd1-cont-text">'+
((__t=( data.game_type ))==null?'':__t)+
'&nbsp;|&nbsp;'+
((__t=( data.size ))==null?'':__t)+
'</div><div class="gd1-cont-text">'+
((__t=( data.desc ))==null?'':__t)+
'</div></div></a>';
 } else if (data.card_type === '2') { 
__p+='<a class="game-downloader gd2" href="'+
((__t=( data.detail ))==null?'':__t)+
'"><img class="gd2-cover" src="'+
((__t=( data.banner ))==null?'':__t)+
'"><div class="gd2-info"><div class="gd-button gd2-btn">'+
((__t=( data.wordsMap.fat[data.installed] ))==null?'':__t)+
'</div><div class="gd2-cont"><div class="gd2-cont-name">'+
((__t=( data.name ))==null?'':__t)+
'</div><div class="gd2-cont-text">'+
((__t=( data.game_type ))==null?'':__t)+
'&nbsp;|&nbsp;'+
((__t=( data.size ))==null?'':__t)+
'</div></div></div></a>';
 } 
__p+='';
}
return __p;
};
    $('tt-game').each(function(_, dom) {
        var params = $.extend({}, dom.dataset);
        params.wordsMap = {
                 //0      1
            thin: ['下载', '打开'],
            fat:  ['立即下载', '进入游戏'],
            font: ['&#xe689;', '&#xe604;']
        };

        var statParams = {
            value: globalArticleObject.statistics.item_id,
            extra: {
                card_type: params.card_type,
                app_name: encodeURIComponent(params.name),
                pkg_name: params.pkg_name,
                app_id: params.app_id,
                app_category: encodeURIComponent(params.game_type),
                media_id: globalArticleObject.author.mediaId
            }
        };

        if (is_iphone() && params.download_url_for_ios) {
            params.installed = 0;
            params.detail = params.download_url_for_ios;

            var $_html = $(render({ data: params }));

            $_html.on('click', function (ev) {
                location.href = params.download_url_for_ios;
            });

            $_html.on('click', function (ev) {
                send_umeng_event('article_card_app_ad', 'click', statParams);
            }).find('.gd-button').on('click', function (ev) {
                send_umeng_event('article_card_app_ad', 'click_download', statParams);
            });

            $(dom).replaceWith($_html);

            send_exposure_event_once($_html.get(0), function () {
                send_umeng_event('article_card_app_ad', 'show', statParams);
            }, true);
        } else if (is_android() && params.download_url_for_android) {
            ToutiaoJSBridge.call("isAppInstalled", {
                pkg_name: params.pkg_name
            }, function (json) {
                params.installed = json.installed == 1 ? 1 : 0;

                if (!params.detail) {
                    params.detail = params.download_url_for_android;
                } else {
                    params.detail = 'sslocal://webview?url=' + encodeURIComponent(params.detail);
                }

                var $_html = $(render({ data: params }));

                $_html.on('click', function (ev) {
                    send_umeng_event('article_card_app_ad', 'click', statParams);
                }).find('.gd-button').on('click', function (ev) {
                    ev.stopPropagation();
                    ev.preventDefault();
                    if (params.installed == 1) {
                        send_umeng_event('article_card_app_ad', 'click_open', statParams);
                        ToutiaoJSBridge.call("openThirdApp", {
                            pkg_name: params.pkg_name
                        }, function (json2) {
                            console.info(json2);
                            if (json2.code == 0) {
                                // TODO 未打开成功，需要提示下载？
                                ToutiaoJSBridge.call('toast', {
                                    text: '打开应用失败，请稍后尝试'
                                });
                            }
                        });
                    } else {
                        send_umeng_event('article_card_app_ad', 'click_download', statParams);
                        location.href = params.download_url_for_android
                    }
                });

                $(dom).replaceWith($_html);

                send_exposure_event_once($_html.get(0), function () {
                    send_umeng_event('article_card_app_ad', 'show', statParams);
                }, true);
            });
        }
    });
}
;

/****************************************************************
 * 图片处理逻辑
 * android头条详情页图片加载js实现
 * https://wiki.bytedance.com/pages/viewpage.action?pageId=10913220
 ****************************************************************/
/**
 * 图片加载失败处理
 * @return {undefined}
 * 可能会被调用的情形
 * none模式，没加载到图时默认情形
 */
function errorimg () {
	var holder = this.parentNode;

	del_elt_class(holder, 'loading');

	if (this.src.indexOf(url_prefix + 'getimage/none/') === 0) {
		// 无图case跟offline有啥必然联系嘞?
		holder.classList.add('offline');
		bindOriginImageLoader(holder);
	} else if (this.src.indexOf(url_prefix + 'image/') === 0) {
		// 本地缓存case
		var p = holder.firstElementChild;
		var spinner = holder.querySelector('.spinner');
		if (p && p.tagName == 'IMG' && p != this && spinner) {
			console.info('p是img还不是<img>???');
			spinner.parentNode.removeChild(spinner);
		}
		del_elt_class(holder, 'offline');
	}
}

/**
 * 点击“小图/无图”加载“origin原图”事件处理函数
 * @param {object} ev 点击事件event对像
 */
function loadOriginImg_handler (ev) {
	ev.preventDefault();
	var that = this;
	setTimeout(function(){
		show_large_image(that);
	}, 100);
}

/**
 * 绑定点击“小图/无图”加载“origin原图”事件
 * @param {object} a 图片包裹元素
 */
function bindOriginImageLoader (a) {
	if (a.getAttribute('ss_href')) {
		a.setAttribute('href', 'javascript:void(0);');
		a.addEventListener('click', loadOriginImg_handler, false);
	}
}

/**
 * 取消绑定点击“小图/无图”加载“origin原图”事件
 * @param {object} a 图片包裹元素
 */
function unbindOriginImageLoader (a) {
	a.removeEventListener('click', loadOriginImg_handler, false);
	if (a.getAttribute('ss_href')) {
		a.setAttribute('href', a.getAttribute('ss_href')); // 不是赋值void了么？
	}
}

/**
 * 从小图/无图状态点击时加载大图
 * @param {object} parent 图片容器
 */
function show_large_image (parent, killCarousel) {
    var isOffline = parent.classList.contains('offline');

    if (isOffline) {
        parent.classList.remove('thumb');
        parent.classList.remove('offline');
    }
    parent.classList.add('loading');

    var allImages = parent.querySelectorAll('img');
    if (allImages.length > 1) {
        return;
    } else if (allImages.length === 1) {
        if (allImages[0].naturalWidth === 0) {
            // remove thumb img if it's not compelete
            parent.removeChild(allImages[0]);
        }
    }

    var index = parent.getAttribute('ss_index');
    var zip_src_path = parent.getAttribute('zip_src_path');

    // 帖子，多图，点击时，直接进入看大图模式，同时加载『小图』
    if (threadGGSwitch && $images.length > 1) {
        !killCarousel && (location.href = parent.getAttribute('ss_href'));
        var src = buildImageSource(threadThumbType, zip_src_path, index);
    } else {
        var src = buildImageSource('origin', zip_src_path, index);
    }

    var tempDiv = document.createElement('div');
    tempDiv.innerHTML = buildImageDOMString(src, 1, 1);
    var tempImg = tempDiv.firstElementChild;
    tempDiv.removeChild(tempImg);
    parent.appendChild(tempImg);

    if (!isOffline) {
        var tempSpinner = document.createElement('i');
        tempSpinner.className = 'spinner';
        parent.appendChild(tempSpinner);
    }

    setTimeout(function () {
        send_request(isOffline ? 'show_image' : 'origin_image', {
            index: index
        });
    }, 1000);

    unbindOriginImageLoader(parent);
}

/**
* 图片加载回调 —— 由客户端调用
* 当img的src以getimage/xxx（xxx取值origin/thumb）等形式进行获取时，客户端先从缓存中获取，失败时会触发img的errorimg事件，
* 但是，此时客户端会进一步去下载该图片，下载成功后通过image_load_cb 修改当前<img>元素。
* 所以，在img的errorimg事件中一定不能手动删除加载失败的<img>节点。
*
* @param {number} i 图片索引
* @param {boolean} ok 是否成功
* @param {boolean} is_large 是否为大图
*/
function image_load_cb(i, ok, is_large) {
	console.info(i, ok, is_large);
	if (i < 0 || i >= $images.length){
		return;
	}

    samePicturesCheckCache.push(i);
    checkSamePictures(i, ok, is_large);

	var parent = $images.get(i),
		img_a = parent.querySelectorAll('img'),
		_this = null;

	if (img_a.length > 0) {
		_this = img_a[0];
	}

	if (! _this){
		return;
	}

	var large_img = null;
	if (img_a.length > 1) {
		large_img = img_a[1];
	}

	var src_path = parent.getAttribute("zip_src_path");
	if (large_img ) {
		if (is_large) {
			if (ok) {
				var src = url_prefix + "image/origin/" + src_path;
				large_img.setAttribute("src", src);
				console.info('m1', src);
			} else {
				parent.removeChild(large_img);
			}
		} else {
			if (ok && _this.src.indexOf("/getimage/") > 0) {
				var src = url_prefix + "image/thumb/" + src_path;
				_this.setAttribute("src", src);
			}
		}
		return;
	}

	if (_this.src.indexOf("/getimage/") < 0){
		return;
	}
    console.info('here', _this.src);

    var p;
	if (_this.src.indexOf("/getimage/origin/") > 0) {
		p = "image/origin/"
	} else if (_this.src.indexOf("/getimage/thumb/") > 0) {
		p = "image/thumb/";
	} else if (_this.src.indexOf('/getimage/'+threadThumbType+'/') > 0) {
        p = 'image/'+threadThumbType+'/';
    } else {
		if (is_large) {
			p = "image/origin/";
		} else {
			p = "image/thumb/";
		}
	}

	var src = url_prefix + p + src_path;
	if (ok) {
		_this.setAttribute("src", src);
	} else {
		del_elt_class(parent, "loading");
	}
};

var samePicturesCheckCache = [];
function checkSamePictures (index, ok, is_large) {
    var $holder = $images.eq(index);
    var holderHref = $holder.attr('href');
    var holderUrl = holderHref.match(/url=[^&]*/);
    if (holderUrl) {
        holderUrl = holderUrl[0];
    } else {
        return;
    }

    $images.each(function (idx, holder) {
        var _href = holder.getAttribute('href');
        if (samePicturesCheckCache.indexOf(idx) === -1 && idx !== index && _href.indexOf(holderUrl) > -1) {
            image_load_cb(idx, ok, is_large);
        }
    });
}

/**
 * 图片加载成功后处理
 * @return {undefined}
 * 可能会被调用的情形
 * none模式下，加载到了已经看过的小图（大图）
 * none模式下，点击默认图加载到大图
 * thumb模式下，正常加载小图（或已经看过的大图）
 * thumb模式下，点击小图加载到大图
 * origin模式下，正常加载大图
 */
function appendimg () {
	var parent = this.parentNode,
		s_w = parent.getAttribute('s_width'),
		s_h = parent.getAttribute('s_height'),
		naturalWidth = this.naturalWidth,
		naturalHeight = this.naturalHeight,
		is_large = false;

	// 加载到的图片是［大图］
	if (threadGGSwitch) {

    } else if (naturalWidth == parent.getAttribute('width') && naturalHeight == parent.getAttribute('height')) {
		is_large = true;

		//适用场景：点小图看大图，大图加载成功，把图片容器调整到［合适尺寸］
        // FIXME 为什么会加载到naturalWidth？？？``
		if (this.getAttribute('width') != s_w) {
            console.info('appendimg_fir', s_w, s_h);
			parent.style.width = s_w + 'px';
			parent.style.height = s_h + 'px';
		}

		del_elt_class(parent, "thumb");
		del_elt_class(parent, "offline");

		unbindOriginImageLoader(parent);
    } else if (naturalWidth > 0 && naturalHeight > 0) {
        // 加载到了图，但是跟原图尺寸不符，这是加载到了thumb图？为什么又进行一次调整？
		var _wh = adjustOriginImageScale(naturalWidth, naturalHeight, maxWidth);
        console.info('appendimg_sec', _wh.w, _wh.h);
		parent.style.width = _wh.w + "px";
		parent.style.height = _wh.h + "px";
	} else {
		// NOTE 会有可能进入这个分支？
		parent.style.width = '120px';
		parent.style.height = '120px';
	}

	if (is_large) {
		toggleGifState(parent, false); // 只有thumb会加gif角标，is_large表明thumb图和原图大小一致
	} else {
		if (this.src.indexOf('getimage/none') > 0) {
			bindOriginImageLoader(parent);
			// TODO 一开始的绑定只给thumb模式加了，但如果看小图看了一部分切出去改成不看图再进来，
			//      还是能看到小图的，getimage/none里的图就是小图，所以此时相当于虽然none模式，
			//      但是有getimage/none图，所以要绑定一次看图事件
			// NOTE 总结一下什么时候需要点小看大
			//      none模式下，没加载到图片（默认图）
			//      none模式下，加载到已缓存小图
			//      thumb模式下，正常加载到图片
			// 所以一开始的绑定其实可以删掉，此处也把none的判断去掉？
		}
	}

	this.style.display = 'block';

	// 确保img是a标签里第一个元素，why?pollImage的时候不是appendChild了吗
	var p = parent.firstElementChild;
	if (p && p.tagName == 'IMG' && p != this) {
		parent.removeChild(p);
	}

	// 删掉spinner
	var spinner = parent.querySelector('.spinner');
	if (spinner) {
		spinner.parentNode.removeChild(spinner);
	}

	del_elt_class(parent, 'loading');
}

/**
 * 调整图片显示尺寸，超过正文宽度的一半就用正文宽度代替
 * @param  {number} originWidth   原始宽度
 * @param  {number} originHeight  原始高度
 * @param  {number} ARTICLE_WIDTH 正文宽度
 * @return {object}               调整后图片尺寸
 */
function adjustOriginImageScale (originWidth, originHeight, ARTICLE_WIDTH) {
	var DEFAULT_ADJUSTED_WIDTH = 200;
	var DEFAULT_ADJUSTED_HEIGHT = 200;
    var originAspectRatio = originWidth / originHeight;
    var adjustedWidth;
    var adjustedHeight;

    if (!originWidth) {
        adjustedWidth = DEFAULT_ADJUSTED_WIDTH;
    } else if (originWidth > ARTICLE_WIDTH / 2) {
        adjustedWidth = ARTICLE_WIDTH;
    } else {
        adjustedWidth = originWidth;
    }

    if (!originAspectRatio) {
        adjustedHeight = DEFAULT_ADJUSTED_HEIGHT;
    } else {
        adjustedHeight = parseInt(adjustedWidth / originAspectRatio);
    }

    return {
        w: adjustedWidth,
        h: adjustedHeight
    };
}

/**
 * 图片容器初始化，调整大、小图尺寸，编制图片索引，设置ss_href
 * @return {undefined}
 *
 * NOTE 尝试使用Zepto的attr/addClass/removeClass方法简化写法，但有10倍左右的性能差距！
 */
function initializeImageContainer (image_load_type) {
    console.info('image_load_type', image_load_type);
	var valid_load_types = ['origin', 'thumb', 'none', 'list640', 'list400', 'list300'];

	// 当图片超过10个时强制开启lazyload
	if (close_lazyload && $images.length > 10) {
	    close_lazyload = false;
		console.info('当图片超过10个时强制开启lazyload');
	}

	// none/thumb模式时，在第一张图前加入［显示大图］按钮
	if (image_load_type !== 'origin' && $images.length > 0 && !threadGGSwitch) {
	    var $button = $('<div class="toggle-img-con">');
	    $button.append('<a class="toggle-img" id="toggle-img" href="javascript:;">显示大图</a>');
	    $button.on('click', function (ev) {
	        $(this).css('visibility', 'hidden');
	        setTimeout(showAllOriginImages, 100);
	    });
	    $images.eq(0).before($button);
		console.info('非大图模式时，在第一张图前加入［显示大图］按钮');
	}

	if (valid_load_types.indexOf(image_load_type) === -1) {
		return;
	}

	// console.time('recalculateImageSize');
	$images.each(function (index, holder) {
		var href = holder.getAttribute('href') || '';
		var zip_src_path = holder.getAttribute('zip_src_path') || '';
		var src = buildImageSource(image_load_type, zip_src_path, index);
		var w, h;

		// 分别计算普通和缩略图尺寸，都需要计算，因为有切换类型的功能
		var s_size = adjustOriginImageScale(holder.getAttribute('width'), holder.getAttribute('height'), maxWidth); //调整后大图显示尺寸
		var t_size = adjustOriginImageScale(holder.getAttribute('thumb_width'), holder.getAttribute('thumb_height'), maxWidth); //调整后thumb图显示尺寸
		holder.setAttribute('s_width', s_size.w);
		holder.setAttribute('s_height', s_size.h);

		holder.setAttribute('t_width', t_size.w);
		holder.setAttribute('t_width', t_size.h);

		// do what?
		holder.setAttribute('ss_index', index);

		// do what?
		if (href && href.indexOf('bytedance://large_image') === 0){
			holder.setAttribute('ss_href', href);
		}

		// do what?
		holder.classList.remove('offline');
		holder.classList.add('loading');

		// 根据图片加载类型，设置相对应的尺寸
		if (threadGGSwitch) {
            var oh = holder.getAttribute('height');
            var ow = holder.getAttribute('width');
            w = h = getThreadImageWidth();
            if (w == innerWidth - 30) {
                h = oh * w / ow;
            }
        } else if (image_load_type === 'thumb') {
			w = t_size.w;
			h = t_size.h;
			holder.classList.add('thumb');
			bindOriginImageLoader(holder);
		} else {
			w = s_size.w;
			h = s_size.h;
		}
		holder.style.width = w + 'px';
		holder.style.height = h + 'px';

		// 如果使用lazyload，暂时只添加对应属性
		// 直接加载则直接将img元素插到a标签中
		if (close_lazyload) {
			holder.innerHTML = buildImageDOMString(src, w, h);
		} else {
			holder.setAttribute('lazy_src', src);
			holder.setAttribute('lazy_w', w);
			holder.setAttribute('lazy_h', h);
		}

		// 只有小图模式需要设置gif标志
		// TODO 应当将toggleGifState里的前置判断挪出来
		if (threadGGSwitch) {
            if (holder.getAttribute('type') === 'gif' || holder.getAttribute('type') === '2') {
                $(holder).append('<i class="forum-image-subscript">GIF</i>');
            } else if ((ow > 3 * oh || oh > 3 * ow) && $images.length !== 1) {
                console.info('hahahahahah')
                $(holder).append('<i class="forum-image-subscript">长图</i>');
            }
        } else if (image_load_type === 'thumb') {
			toggleGifState(holder, true);
		}

		$(holder).on('click', function(ev) {
			var _href = holder.getAttribute('href');
            var _rl = holder.getAttribute('redirect-link');
            if (typeof _rl === 'string' && _rl.indexOf('sslocal') > -1) {
                ev.preventDefault();
                location.href = _rl;
            } else if (_href.indexOf('bytedance://large_image') > -1) {
				if (_href.indexOf('&left=') > -1) {
					_href = _href.substring(0, _href.indexOf('&left='));
				}
				var _coords = $(holder).offset();
				var surfix = '&left=' + _coords.left + '&top=' + _coords.top + '&width=' + (_coords.width) + '&height=' + (_coords.height);
				holder.setAttribute('href', _href+surfix);
                send_umeng_event('talk_detail', 'picture_click', globalArticleObject.forumStatisticsParams);
			}
		})
	});
	// console.timeEnd('recalculateImageSize');
}

/**
 * 切换到显示大图模式，其实目的只是修改显示尺寸、修改图片src
 * @param {string} image_load_type 图片加载模式
 * @return {undefined}
 */
function changeImageLoadType (image_load_type) {
	$images.each(function(index, holder){
		var zip_src_path = holder.getAttribute('zip_src_path');
		var src = buildImageSource(image_load_type, zip_src_path, index);
		var h = holder.getAttribute('s_height');
		var w = holder.getAttribute('s_width');

		// 修改图片尺寸为［大图］尺寸
		// TODO 修改为带参数的方法时应该对此加以纠正
		console.info('changeImageLoadType', w, h);
		holder.style.width = w + 'px';
		holder.style.height = h + 'px';

		// change的时候，
		// 先看是不是lazy开了，
		// 如果关了，直接塞新的图片就行
		// 开着的话，如果已经加载过的图片，就替换它的地址
		// 没有加载过的更新它的信息
		if (close_lazyload || !holder.getAttribute('lazy_src')) {
			holder.innerHTML = buildImageDOMString(src, w, h);
		} else {
			holder.setAttribute('lazy_src', src);
			holder.setAttribute('lazy_w', w);
			holder.setAttribute('lazy_h', h);
		}
	});
}

/**
 * 点击［显示大图］按钮，按大图模式加载所有图片
 * @return {undefined}
 */
function showAllOriginImages () {
	changeImageLoadType('origin');

	if (!close_lazyload) {
		initializeImageLazyload();
	}

	// 通知客户端点击事件
	setTimeout(function(){
		window.location.href = 'bytedance://toggle_image?action=show';
	}, 500);

}

/**
 * 图片懒加载初始化，收集所有待加载图片，调用轮询图片加载方法
 * @return {undefined}
 */
function initializeImageLazyload () {
	lazyloadHolders = Array.prototype.slice.call(document.querySelectorAll('[lazy_src]')); // 如果小图lazy过，切换大图时，已经加载的图片就行了？
	_pollImages();
	document.removeEventListener('scroll', _pollImages, false);
	document.addEventListener('scroll', _pollImages, false);
}

/**
 * 构造一个img标签
 * @param  {string} src    图片路径
 * @param  {number} width  宽
 * @param  {number} height 高
 * @return {undefined}
 */
function buildImageDOMString (src, width, height) {
	return '<img src="' + src + '" width="' + width + '" height="' + height + '" style="display: none;" onload="appendimg.call(this);" onerror="errorimg.call(this);" />'
}
function initbuildImageSource () {
	var group_id = getMeta('group_id') || 0;
	window.buildImageSource = function (type, path, index) {
		return url_prefix + 'getimage/' + type + '/' + path + '/' + group_id + '/' + index;
	}
}

/**
 * 轮询图片，将inview的
 */
function _pollImages () {
	if (lazyloadHolders.length === 0) {
		document.removeEventListener('scroll', _pollImages, false);
		return;
	}
	lazyloadHolders.forEach(function(holder, i) {
		if (holder && isElementinViewport(holder)) {
			var w = holder.getAttribute('lazy_w');
			var h = holder.getAttribute('lazy_h');
			var src = holder.getAttribute('lazy_src');
			holder.innerHTML = holder.innerHTML + buildImageDOMString(src, w, h); // TODO append
			holder.removeAttribute('lazy_src'); // 防止被再次初始化
			// lazyloadHolders.splice(i, 1); // 处理完毕即删除
		}
	});
	lazyloadHolders = Array.prototype.slice.call(document.querySelectorAll('[lazy_src]'));
}

/**
 * 判断图片是否处于屏幕可视区域，在屏幕以下offset处即触发懒加载，与视频的判断差别
 * @param {object} element 图片容器DOM
 * @return {undefined}
 */
function isElementinViewport (element) {
	var offset = 100; //图片lazyload默认偏移量
	var coords = element.getBoundingClientRect();
	if (coords.top < 0) {
		return true;
	} else {
		return ((coords.top >= 0 && coords.left >= 0 && coords.top) <= (window.innerHeight || document.body.clientHeight) + offset);
	}
}

/**
* gif图片增加特定标识
* @param {object} holder 图片容器
* @param {boolean} show 是否需要展示gif标识
*/
function toggleGifState(holder, show){
	if(!holder) return;
	if(holder.getAttribute("type") !== 'gif' && holder.getAttribute("type") !== '2') return;

	var gif_play  = holder.querySelector(".gif_play");
	if (show) {
		if (!gif_play) {
			gif_play = document.createElement("i");
			gif_play.className = 'gif_play';
			holder.appendChild(gif_play);
		}
	} else {
		if(gif_play){
			gif_play.parentNode.removeChild(gif_play);
		}
	}
}
/****************************************************************
 * 视频处理逻辑
 ****************************************************************/
/**
 * 调用客户端播放器进行视频播放
 * @param {object} video 视频容器<div class="custom-video">
 * @param {number} status 区分是用户点击视频，还是视频自动播放，其中 0 代表点击播放，1 代表自动播放
 */
function playVideo (video, status) {
	var v_size = video.getAttribute('data-video-size');
	var coords = video.getBoundingClientRect();
	var frame = [coords.left, video.offsetTop, 640, 435];

	if (v_size) {
		var obj = null;
		try {
			obj = JSON.parse(v_size);
			if (obj && obj.normal && obj.normal.h && obj.normal.w) {
				frame[2] = obj.normal.w;
				frame[3] = obj.normal.h;
			}
		} catch (ex) {
			// 防止JSON解析出错?
		}
	}

	window.ToutiaoJSBridge.call('playNativeVideo', {
		sp: video.getAttribute('data-sp'),
		vid: video.getAttribute('data-vid'),
		frame: frame,
		status: status
	}, function (event) {
		// 先吧所有视频展现出来。多个视频的时候连续点会出问题
		$('.custom-video').show();
		// 调用客户端播放器时的回调函数，调整视频播放时位置（插入到页面顶部，正文中视频暂时隐藏）
		if (event.code == 1) {
			var evideo = document.querySelector('[data-vid="' + event.vid + '"]');
			if (evideo) {
				evideo.style.display = 'none';
				$(evideo).next('.cv-wd-info-wrapper').hide();
				document.body.style.marginTop = event.height + 'px';
			}
		}
	});
}
/****************************************************************
 * 初始化页面入口
 ****************************************************************/
/**
 * 全局变量
 */
// 客户端传参识别，close_lazyload 和 url_prefix可由客户端在<head>内插入<script></script>代码来定义
if (typeof close_lazyload === 'undefined') {
    close_lazyload = false;
}
if (typeof url_prefix === 'undefined') {
    url_prefix = 'content://com.ss.android.article.base.ImageProvider/';
}

var $images = [];
var lazyloadHolders = []; //lazyload剩余图片库存
var maxWidth;
var threadGGSwitch = globalArticleObject.article.type === 'forum' && forum_extra.use_9_layout; // 符合条件版本的帖子页，图片按宫格显示

function initCustomStyle () {
	var font_size = getMeta('font_size') || 'm';
	var day_mode = getMeta('night_mode') ? 0 : 1; // night_mode取反给day_mode

	setDayMode(day_mode);
	setFontSize(font_size);
}

var threadThumbType = 'list640';
var getThreadImageWidth = function() {
    var $threadImages = $('.image');
    var threadImagesCount = $threadImages.length;

    var containerWidth = innerWidth - 30;
    var MARGIN = 4;
    var width;

    if (threadImagesCount === 1) {
        $('body').attr('images', 1);
        width = containerWidth;
    } else if (threadImagesCount === 2 || threadImagesCount === 4) {
        $('body').attr('images', 4);
        width = Math.floor((containerWidth - MARGIN) / 2);
        threadThumbType = 'list400';
    } else {
        $('body').attr('images', 9);
        width = Math.floor((containerWidth - MARGIN * 2) / 3);
        threadThumbType = 'list300';
    }

    $('body').attr('newimage', 'true');
    getThreadImageWidth = function() {
        return width; // 内存泄露吧！
    }
}

function initPage () {
	$images = $('.image');
	maxWidth = document.getElementsByTagName('article')[0].offsetWidth || 320;

	initCustomStyle();
	initbuildImageSource(); // 就为了少一个全局变量容易么

	// 延迟计算图片尺寸，防止部分机型渲染缓慢
	setTimeout(function(){
		initScrollEvents();
		var img_type = getMeta('load_image') || 'origin';
		if ('is_gallery' in h5_extra && !!h5_extra.is_gallery) {
			img_type = 'origin';
		}
        if (threadGGSwitch) {
            getThreadImageWidth();
            if (img_type === 'none') {
                lazy_load = true;
            } else {
                if ($images.length === 1) {
                    img_type = 'origin';
                } else {
                    img_type = threadThumbType;
                    close_lazyload = true;
                }
            }
        }
		// if (img_type === 'origin' && globalArticleObject.h5_settings.is_show_concern_guide_picture) {
		// 	$('#concern-guide-picture').attr('show', 'true');
		// }

        if (!threadGGSwitch) {
            $('.image').each(function (index, holder) {
                var $holder = $(holder);
                var $parent = $holder.parent();

                if ($parent.is('p')) {
                    if ($parent.text() !== '') {
                        console.info('['+index+']所在段落有其他内容，分割');
                        $holder.wrap('<span class="image-wrap">');
                    } else if ($parent.find('.image').length > 0) {
                        console.info('['+index+']所在段落有其他图片，应当分割');
                        $holder.wrap('<span class="image-wrap">');
                    } else {
                        console.info('['+index+']正确');
                        $parent.addClass('image-wrap');
                    }
                } else {
                    console.info('['+index+']直接加包裹');
                    $holder.wrap('<p class="image-wrap">');
                }
            });
        }

		initializeImageContainer(img_type);
		if (!close_lazyload) {
			initializeImageLazyload();
		}
	}, 100);

/* 以下部分两端已经统一逻辑 */
	bindStatisticsEvents();
    processParagraph();

    if (globalArticleObject.article.type === 'forum') {
        processThreadRepostContent();
        processRepostThread();
        processRepostArticle();
        processFilm();
    }

    processZZComments();
	processCustomVideo();
	processTable(true);
	processAudio();
	processDigg();
	processProHref();
	processNovel();
	processPgcBottomCard();
	processPhoneGroup();
    processGameDownloader();

	setTimeout( function () {
        checkHeaderDisplayed();
		document.addEventListener('scroll', checkHeaderDisplayed, false);
		if ('wenda_extra' in window) {
			if (wenda_extra.wd_version >= 3 && wenda_extra.showMode != 1) {
				document.addEventListener('scroll', checkWendaABHeaderLayout, false);
			} else if (wenda_extra.wd_version >= 1) {
				document.addEventListener('scroll', checkWendanextDisplayed, false);
			}
		}
	}, 100);

	window.ToutiaoJSBridge.on('page_state_change', processPageStateChangeEvent);

	//服务端information接口先于此脚本加载完毕时，服务端提供videoAutoPlayCallback供当前脚本加载完毕后调用，保险措施
	if (typeof videoAutoPlayCallback == 'function') {
		videoAutoPlayCallback();
	}

	//服务端information接口先于此脚本加载完毕时，服务端提供updateAppreciateCountByServerCallback供当前脚本加载完毕后调用，保险措施
	if (typeof updateAppreciateCountByServerCallback == 'function') {
		updateAppreciateCountByServerCallback();
	}
/* 以上部分两端已经统一逻辑 */

	//通知客户端domReady
	location.href = "bytedance://domReady";
	// android的webview无法获取html的真实高度
	tellClientContentSize();

	(function () {
	    // NOTE android要对小说、问答（新版）做特殊处理
	    //      每隔500毫秒检测"目录"是否被添加到页面上
	    //      一旦添加上，再过500毫秒通知客户端其距离底部的位置
	    //
	    // FIXME 问答是否展示目录是在infor接口，拿到.serial不一定是show的
		if ('novel_data' in window.h5_extra || ('wenda_extra' in window && wenda_extra.wd_version >= 1)) {
			var _timer = setInterval(function () {
	            if (document.querySelector('.serial')) { // FIXME 是否应当改为获取其尺寸更为准确？
	                                                     //       或采用MutationObserver?
				    var ot = document.querySelector('.serial').offsetTop; // "确保"展示
					if (ot > 0) {
						clearInterval(_timer);
		                setTimeout(function(){
		                    ToutiaoJSBridge.call('onGetSeriesLinkPosition', {
		                        value: document.body.scrollHeight - ot
		                    });
		                }, 500);
					}
	            }
	        }, 500);
		}
	})();
}
document.addEventListener("DOMContentLoaded",initPage,false);
/****************************************************************
 * android端对webview内容高度的获取方法，真是造孽啊！
 *****************************************************************/
function tellClientContentSize () {
    var $footer = $('footer');
    var lastHeight = 0;
    setInterval(function(){
        var footerCoords = $footer.offset();
        var height = footerCoords.top + footerCoords.height;
        if (height !== lastHeight) {
            lastHeight = height;
            ToutiaoJSBridge.call('webviewContentResize', {
                height: height
            });
        }
    }, 200);
}
/****************************************************************
* 部分DOM操作基本封装，后续版本择机完成zepto改造后，废弃以下函数
*****************************************************************/
function del_elt_class(elt, cls) {
	if (!elt || !cls)
		return false;
	try {
		var clazz = elt.getAttribute("class");
		if (clazz == null)
			return false;
		var has = false;
		var clazzes = clazz.split(" ");
		clazz = ""
		for (var i = 0; i < clazzes.length; i++) {
			if (clazzes[i] == cls) {
				has = true;
				continue;
			} else {
				clazz = clazz.concat(" ", clazzes[i]);
			}
		}
		if (has) {
			elt.setAttribute("class", clazz);
			return true;
		}
	} catch (e) {
	}
	return false;
}
/**
* 安卓客户端的一些注意事项：
* 1，据说某些安卓设备（如4.0.x, 4.1.x）在点击后立即进行大量运算，会有crash情形，
* 所以，本文中看似一些多余的超时调用大多基于这个原因而设。
* 2，安卓版本的图片加载逻辑依然晦涩难懂，待优化
*/
/**
 * 功能不知，貌似是传递什么统计
 */
function initScrollEvents(){
	var doubleinnerHeight = innerHeight*2,
		scrollOneScreen = document.height <= innerHeight,
		scrollDoubleScreen = document.height <= doubleinnerHeight,
		scrollBottom = false;

	if(!scrollBottom && document.height <= innerHeight){
		window.location.href = "bytedance://finish_content";
		scrollBottom = true;
	}else{
		window.onscroll = function(){
			if(!scrollOneScreen && (scrollY > innerHeight)){
				//console.log("滚过一屏");
				window.location.href = "bytedance://read_content";
				scrollOneScreen = true;
			};

			if(!scrollDoubleScreen && (scrollY > doubleinnerHeight)){
				//console.log("滚过两屏");
				window.location.href = "bytedance://finish_content";
				scrollDoubleScreen = true;
			};

			//修复部分android不能自动加载的问题
			if(!scrollDoubleScreen && !scrollBottom && scrollY > document.height - innerHeight - 5){
				window.location.href = "bytedance://finish_content";
				scrollBottom = true;
			};
		}
	}
}

// 客户端轮播图浏览时切换动作，可以『认为』此时已有指定index的origin图，那么：
// 大图模式下，请求此图即可
// 小图模式下，可展示/切换为大图
// 无图模式下，请求此图即可
//
// 针对帖子有所不同，无论何种模式
// 只有一图时，请求大图（悖论，此时必然已经加载了图才会进入轮播）
// 多图时，请求小图
function onCarouselImageSwitch (index) {
    console.info('onCarouselImageSwitch', index);
    // if (threadGGSwitch && $images.length > 1) {
    //     // 加载一张640图
    // } else {
    //     // 加载一张大图
    // }
    show_large_image($images.get(index), true);
}
